/*
 * Program:	Berkeley mail (BEZERK) definitions
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	20 December 1989
 *
 * Copyright 1989, 1990, 1991 by University of Washington.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */

/* Build parameters */

#define LOCKTIMEOUT 5		/* lock timeout in minutes */


/* Command bits from bezerk_getflags() */

#define fSEEN 1
#define fDELETED 2
#define fFLAGGED 4
#define fANSWERED 8


/* Status string */

#define STATUS "Status: %sO\nX-Status: %s%s%s\n\n"

/* BEZERK per-message cache information */

typedef struct file_cache {
  long internal;		/* pointer to internal header */
  long header;			/* pointer to RFC 822 header */
  long headersize;		/* size of RFC 822 header */
  long body;			/* pointer to message body */
  long bodysize;		/* size of message body */
  int date;			/* integer representation of date */
  char status[sizeof (STATUS)];	/* status flags */
} FILECACHE;


/* BEZERK I/O stream local data */
	
typedef struct bezerk_local {
  char *host;			/* local host name */
  char *fname;			/* local copy of file name */
  int ld;			/* lock file descriptor */
  char *lname;			/* lock file name */
  off_t filesize;		/* file size parsed */
  time_t filetime;		/* last file time */
  FILECACHE *msgs[MAXMESSAGES];	/* pointers to message-specific information */
  char *text;			/* mailbox text buffer */
  long textsize;		/* size of mailbox text buffer */
  long textend;			/* end of text in mailbox text buffer */
  char *lsjumb;			/* the Incomparables */
  char *buf;			/* temporary buffer */
  long buflen;			/* current size of temporary buffer */
} BEZERKLOCAL;


/* Convenient access to local data */

#define LOCAL ((BEZERKLOCAL *) stream->local)

/* Function prototypes */

DRIVER *bezerk_valid (char *name);
MAILSTREAM *bezerk_open (MAILSTREAM *stream);
void bezerk_close (MAILSTREAM *stream);
void bezerk_fetchfast (MAILSTREAM *stream,char *sequence);
void bezerk_fetchflags (MAILSTREAM *stream,char *sequence);
ENVELOPE *bezerk_fetchenvelope (MAILSTREAM *stream,int msgno);
char *bezerk_snarf (MAILSTREAM *stream,int msgno,int *size);
char *bezerk_fetchheader (MAILSTREAM *stream,int msgno);
char *bezerk_fetchtext (MAILSTREAM *stream,int msgno);
char *bezerk_fetchbody (MAILSTREAM *stream,int msgno,int sec);
void bezerk_setflag (MAILSTREAM *stream,char *sequence,char *flag);
void bezerk_clearflag (MAILSTREAM *stream,char *sequence,char *flag);
void bezerk_search (MAILSTREAM *stream,char *criteria);
MAILSTREAM *bezerk_ping (MAILSTREAM *stream);
void bezerk_check (MAILSTREAM *stream);
void bezerk_expunge (MAILSTREAM *stream);
void bezerk_copy (MAILSTREAM *stream,char *sequence,char *mailbox);
void bezerk_move (MAILSTREAM *stream,char *sequence,char *mailbox);

void bezerk_abort (MAILSTREAM *stream);
char *bezerk_file (char *dst,char *name);
int bezerk_lock (char *file,int flags,int mode,char *lock,int op);
void bezerk_unlock (int fd,char *file);
int bezerk_parse (MAILSTREAM *stream,char *lock);
void bezerk_save (MAILSTREAM *stream,int fd);
int bezerk_writev (int fd,struct iovec *iov,int iovc);
void bezerk_copy_messages (MAILSTREAM *stream,unsigned char s[],char *mailbox);
void bezerk_write_message (struct iovec iov[],int *i,char *s,FILECACHE *m);
void bezerk_update_status (char *status,MESSAGECACHE *elt);
short bezerk_getsequence (MAILSTREAM *stream,unsigned char s[],char *sequence);
short bezerk_getflags (MAILSTREAM *stream,char *flag);
char bezerk_search_all (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_answered (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_deleted (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_flagged (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_keyword (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_new (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_old (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_recent (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_seen (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_unanswered (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_undeleted (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_unflagged (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_unkeyword (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_unseen (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_before (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_on (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_since (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_body (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_subject (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_text (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_bcc (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_cc (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_from (MAILSTREAM *stream,int msgno,char *d,int n);
char bezerk_search_to (MAILSTREAM *stream,int msgno,char *d,int n);
char (*bezerk_search_date
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,int n),int *n))
     (MAILSTREAM *stream,int msgno,char *d,int n);
int bezerk_date (char *s);
char (*bezerk_search_flag
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,int n),char **d))
     (MAILSTREAM *stream,int msgno,char *d,int n);
char (*bezerk_search_string
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,int n),char **d,int *n))
     (MAILSTREAM *stream,int msgno,char *d,int n);
