/*
 * Program:	Interactive Mail Access Protocol 2 (IMAP2) definitions
 *
 * Author(s):	Mark Crispin
 *		Symbolic Systems Resources Group
 *		Knowledge Systems Laboratory
 *		Departments of Computer Science and Medicine
 *		Stanford University
 *		Stanford, CA  94305
 *
 * Date:	15 June 1988
 *
 * Sponsorship:	This work was supported by the Biomedical Research
 *		Technology Program of the National Institutes of Health
 *		under grant number RR-00785.
 *
 * Copyright 1988, 1989, 1990, 1991 by The Leland Stanford Junior University.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license from the Stanford Office of
 * Technology Licensing.
 *
 */


/* Build parameters */

#define MAXLOGINTRIALS 3	/* maximum number of login trials */
#define MAPLOOKAHEAD 20		/* fetch lookahead */


/* Constants */

#define IMAPTCPPORT 143		/* assigned TCP contact port */

/* IMAP2 specific definitions */


/* Parsed reply message from imap_reply */

typedef struct imap_parsed_reply {
  char *line;			/* original reply string pointer */
  char *tag;			/* command tag this reply is for */
  char *key;			/* reply keyword */
  char *text;			/* subsequent text */
} IMAPPARSEDREPLY;


/* IMAP2 I/O stream local data */
	
typedef struct imap_local {
  void *tcpstream;		/* TCP I/O stream */
  IMAPPARSEDREPLY reply;	/* last parsed reply */
  unsigned int use_encoding : 1;/* server supports encodings */
  unsigned int use_find : 1;	/* server supports FIND command */
  unsigned int use_bboard : 1;	/* server supports BBOARD command */
  char *header[MAXMESSAGES];	/* cached message headers */
  char *text[MAXMESSAGES];	/* cached message RFC822 texts */
} IMAPLOCAL;


/* Convenient access to local data */

#define LOCAL ((IMAPLOCAL *) stream->local)


/* Helper macro to take an action and return if two strings are equal */

#define IFEQUAL(S1,S2,ACTION) if (!strcmp (S1,S2)) { ACTION; return; }

/* Coddle certain compilers' 6-character symbol limitation */

#ifdef __COMPILER_KCC__
#define map_valid ivalid
#define map_open iopen
#define map_close iclose
#define map_fetchfast iffast
#define map_fetchflags ifflags
#define map_fetchenvelope ifenv
#define map_fetchheader ifhead
#define map_fetchtext iftext
#define map_fetchbody ifbody
#define map_setflag isflag
#define map_clearflag icflag
#define map_search isearch
#define map_ping iping
#define map_check icheck
#define map_expunge iexpun
#define map_copy icopy
#define map_move imove

#define imap_hostfield imhfld
#define imap_mailboxfield immfld
#define imap_host imhost
#define imap_select imsele
#define imap_send imsend
#define imap_reply imrepl
#define imap_parse_reply imprep
#define imap_fake imfake
#define imap_OK imok
#define imap_parse_unsolicited impuns
#define imap_parse_flaglst impflg
#define imap_searched imsear
#define imap_expunged imexpu
#define imap_parse_data impdat
#define imap_parse_prop imppro
#define imap_parse_envelope impenv
#define imap_parse_adrlist impadl
#define imap_parse_address impadr
#define imap_parse_flags impfla
#define imap_parse_sys_flag impsfl
#define imap_parse_user_flag impufl
#define imap_parse_string impstr
#define imap_parse_number impnum
#define imap_parse_enclist impecl
#define imap_parse_encoding impenc
#define imap_parse_body impbod
#endif

/* Function prototypes */

DRIVER *map_valid (char *name);
MAILSTREAM *map_open (MAILSTREAM *stream);
void map_close (MAILSTREAM *stream);
void map_fetchfast (MAILSTREAM *stream,char *sequence);
void map_fetchflags (MAILSTREAM *stream,char *sequence);
ENVELOPE *map_fetchenvelope (MAILSTREAM *stream,int msgno);
char *map_fetchheader (MAILSTREAM *stream,int msgno);
char *map_fetchtext (MAILSTREAM *stream,int msgno);
char *map_fetchbody (MAILSTREAM *stream,int msgno,int section);
void map_setflag (MAILSTREAM *stream,char *sequence,char *flag);
void map_clearflag (MAILSTREAM *stream,char *sequence,char *flag);
void map_search (MAILSTREAM *stream,char *criteria);
MAILSTREAM *map_ping (MAILSTREAM *stream);
void map_check (MAILSTREAM *stream);
void map_expunge (MAILSTREAM *stream);
void map_copy (MAILSTREAM *stream,char *sequence,char *mailbox);
void map_move (MAILSTREAM *stream,char *sequence,char *mailbox);

char *imap_hostfield (char *string);
char *imap_mailboxfield (char *string);
char *imap_host (MAILSTREAM *stream);
IMAPPARSEDREPLY *imap_send (MAILSTREAM *stream,char *command,char *args);
IMAPPARSEDREPLY *imap_reply (MAILSTREAM *stream,char *tag);
IMAPPARSEDREPLY *imap_parse_reply (MAILSTREAM *stream,char *text);
IMAPPARSEDREPLY *imap_fake (MAILSTREAM *stream,char *tag,char *text);
int imap_OK (IMAPPARSEDREPLY *reply);
void imap_parse_unsolicited (MAILSTREAM *stream,IMAPPARSEDREPLY *reply);
void imap_parse_flaglst (MAILSTREAM *stream,IMAPPARSEDREPLY *reply);
void imap_searched (MAILSTREAM *stream,char *text);
void imap_expunged (MAILSTREAM *stream,int msgno);
void imap_parse_data (MAILSTREAM *stream,int msgno,char *text,
		      IMAPPARSEDREPLY *reply);
void imap_parse_prop (MAILSTREAM *stream,MESSAGECACHE *elt,char *prop,
		      char **txtptr,IMAPPARSEDREPLY *reply);
void imap_parse_envelope (MAILSTREAM *stream,MESSAGECACHE *elt,char **txtptr,
			       IMAPPARSEDREPLY *reply);
ADDRESS *imap_parse_adrlist (MAILSTREAM *stream,char **txtptr,
			     IMAPPARSEDREPLY *reply);
ADDRESS *imap_parse_address (MAILSTREAM *stream,char **txtptr,
			     IMAPPARSEDREPLY *reply);
void imap_parse_flags (MAILSTREAM *stream,MESSAGECACHE *elt,char **txtptr);
void imap_parse_sys_flag (MESSAGECACHE *elt,char *flag);
void imap_parse_user_flag (MAILSTREAM *stream,MESSAGECACHE *elt,char *flag);
char *imap_parse_string (MAILSTREAM *stream,char **txtptr,
			 IMAPPARSEDREPLY *reply);
unsigned long imap_parse_number (char **txtptr);
void imap_parse_enclist (MESSAGECACHE *elt,char **txtptr);
BODY *imap_parse_encoding (char **txtptr);
void imap_parse_body (MAILSTREAM *stream,MESSAGECACHE *elt,char *seg,
		      char **txtptr,IMAPPARSEDREPLY *reply);
