/*
 * Program:	Mail library test program (MTEST)
 *
 * Author(s):	Mark Crispin
 *		Symbolic Systems Resources Group
 *		Knowledge Systems Laboratory
 *		Departments of Computer Science and Medicine
 *		Stanford University
 *		Stanford, CA  94305
 *
 * Date:	8 July 1988
 *
 * Sponsorship:	This work was supported by the Biomedical Research
 *		Technology Program of the National Institutes of Health
 *		under grant number RR-00785.
 *
 * Copyright 1988, 1989, 1990, 1991 by The Leland Stanford Junior University.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license from the Stanford Office of
 * Technology Licensing.
 *
 */


#include <stdio.h>
#include <ctype.h>
#ifdef __STDC__
#include <string.h>
#else
#include "ansi.h"
#endif
#include <sys/types.h>
#include "mail.h"
#include "smtp.h"
#include "misc.h"
#include "osdep.h"
#if unix
#include <pwd.h>
#include <netdb.h>
#endif


char selected[MAXMESSAGES];	/* T if message selected */
char *curhst = NIL;		/* currently connected host */
char *curusr = NIL;		/* current login user */
char personalname[256];		/* user's personal name */
extern DRIVER imapdriver,bezerkdriver,tenexdriver;
#ifdef __MSDOS__
extern _stklen = 65535;		/* larger stack for DOS */
#endif

static char *hostlist[] = {	/* SMTP server host list */
  "mailhost",
  "localhost",
  NIL
};
void main ();
void mm (MAILSTREAM *stream,short debug);
void header (MAILSTREAM *stream,int msgno);
void status (MAILSTREAM *stream);
void prompt (char *msg,char *txt);
void smtptest (short debug);

/* Main program - initialization */

void main ()
{
  MAILSTREAM *stream = NIL;
  char tmp[MAILTMPLEN];
  short debug;
#if unix
  char *name;
  char *suffix;
  struct passwd *pwd;
  struct hostent *host_name;
  gethostname (tmp,MAILTMPLEN);	/* get local name */
				/* get it in full form */
  curhst = (host_name = gethostbyname (tmp)) ?
    cpystr (host_name->h_name) : cpystr (tmp);
				/* get user name and passwd entry */
  if (name = (char *) getlogin ()) pwd = getpwnam (name);
  else {
    pwd = getpwuid (getuid ());	/* get it this way if detached, etc */
    name = pwd->pw_name;
  }
  curusr = cpystr (name);	/* current user is this name */
  strcpy (tmp,pwd->pw_gecos);	/* probably not necessay but be safe */
				/* dyke out the office and phone poop */
  if (suffix = strchr (tmp,',')) suffix[0] = '\0';
  strcpy (personalname,tmp);	/* make a permanent copy of it */
#else
  prompt ("Personal name: ",personalname);
#endif
  printf ("MTest -- C client test program\n");
  mail_link (&imapdriver);	/* link in IMAP driver */
#ifndef __MSDOS__
#ifndef macintosh
  mail_link (&tenexdriver);	/* link in Tenex mail driver */
#endif
#endif
#if unix
  mail_link (&bezerkdriver);	/* link in Berkeley mail driver */
#endif
				/* user wants protocol telemetry? */
  prompt ("Debug protocol (y/n)?",tmp);
  ucase (tmp);
  debug = (tmp[0] == 'Y') ? T : NIL;
  do prompt ("Mailbox: ",tmp);	/* get mailbox to open */
  while (!(stream = mail_open (stream,tmp,debug)));
  mm (stream,debug);		/* run user interface if opened */
#ifdef __MSDOS__
  netshut ();			/* clean up */
#endif
}

/* MM command loop
 * Accepts: MAIL stream
 */

void mm (MAILSTREAM *stream,short debug)
{
  char cmd[256];
  char *arg;
  int i;
  int last;
  last = 0;
  status (stream);		/* first report message status */
  while (T) {
    prompt ("MTest>",cmd);	/* prompt user, get command */
				/* get argument */
    if (arg = strchr (cmd,' ')) *arg++ = '\0';
    switch (*ucase (cmd)) {	/* dispatch based on command */
      case 'C':			/* Check command */
	mail_check (stream);
	status (stream);
	break;
      case 'D':			/* Delete command */
	if (arg) last = atoi (arg);
	else {
	  if (last == 0 ) {
	    printf ("?Missing message number\n");
	    break;
	  }
	  arg = cmd;
	  sprintf (arg,"%d",last);
	}
	if (last > 0 && last <= stream->nmsgs)
	  mail_setflag (stream,arg,"\\DELETED");
	else printf ("?Bad message number\n");
	break;
      case 'E':			/* Expunge command */
	mail_expunge (stream);
	last = 0;
	break;
      case 'H':			/* Headers command */
	if (arg) {
	  if (!(last = atoi (arg))) {
	    memset (selected,NIL,stream->nmsgs);
	    mail_search (stream,arg);
	    for (i = 0; i < stream->nmsgs; ++i)
	      if (selected[i]) header (stream,i+1);
	    break;
	  }
	}
	else if (last == 0) {
	  printf ("?Missing message number\n");
	  break;
	}
	if (last > 0 && last <= stream->nmsgs) header (stream,last);
	else printf ("?Bad message number\n");
	break;
      case 'N':			/* New mailbox command */
	if (!arg) {
	  printf ("?Missing mailbox\n");
	  break;
	}
				/* get the new mailbox */
	while (!(stream = mail_open (stream,arg,debug)))
	  prompt ("Mailbox: ",arg);
	last = 0;
	status (stream);
	break;
      case 'Q':			/* Quit command */
	mail_close (stream);
	return;
      case 'S':			/* Send command */
	smtptest (debug);
	break;
      case 'T':			/* Type command */
	if (arg) last = atoi (arg);
	else if (last == 0 ) {
	  printf ("?Missing message number\n");
	  break;
	}
	if (last > 0 && last <= stream->nmsgs) {
	  fputs (mail_fetchheader (stream,last),stdout);
	  puts (mail_fetchtext (stream,last));
	}
	else printf ("?Bad message number\n");
	break;
      case 'U':			/* Undelete command */
	if (arg) last = atoi (arg);
	else {
	  if (last == 0 ) {
	    printf ("?Missing message number\n");
	    break;
	  }
	  arg = cmd;
	  sprintf (arg,"%d",last);
	}
	if (last > 0 && last <= stream->nmsgs)
	  mail_clearflag (stream,arg,"\\DELETED");
	else printf ("?Bad message number\n");
	break;
      case 'X':			/* Xit command */
	mail_expunge (stream);
	mail_close (stream);
	return;
      case '+':
	mail_debug (stream); debug = T;
	break;
      case '-':
	mail_nodebug (stream); debug = NIL;
	break;
      case '?':			/* ? command */
	printf ("Check, Delete, Expunge, Headers, New Mailbox,"
		" Quit, Send, Type, Undelete, Xit,\n"
		"+, -, or <RETURN> for next message\n");
	break;
      case '\0':		/* null command (type next message) */
	if (last > 0 && last++ < stream->nmsgs) {
	  fputs (mail_fetchheader (stream,last),stdout);
	  puts (mail_fetchtext (stream,last));
	}
	else printf ("%%No next message\n");
	break;
      default:			/* bogus command */
	printf ("?Unrecognized command: %s\n",cmd);
	break;
    }
  }
}

/* MM display header
 * Accepts: IMAP2 stream
 *	    message number
 */

void header (MAILSTREAM *stream,int msgno)
{
  long i;
  char tmp[256];
  char tmpx[20];
  MESSAGECACHE *cache = mail_elt (stream,msgno);
  mail_fetchenvelope (stream,msgno);
  tmp[0] = cache->recent ? (cache->seen ? 'R': 'N') : ' ';
  tmp[1] = (cache->recent | cache->seen) ? ' ' : 'U';
  tmp[2] = cache->flagged ? 'F' : ' ';
  tmp[3] = cache->answered ? 'A' : ' ';
  tmp[4] = cache->deleted ? 'D' : ' ';
  sprintf (tmp+5,"%4d) ",cache->msgno);
  strncpy (tmp+11,cache->internal_date ? cache->internal_date : "dd-mmm",6);
  tmp[17] = ' ';
  tmp[18] = '\0';
  strcat (tmp,mail_fetchfrom (stream,msgno,20));
  strcat (tmp," ");
  if (i = cache->user_flags) {
    strcat (tmp,"{");
    while (i) {
      strcat (tmp,stream->user_flags[find_rightmost_bit (&i)]);
      if (i) strcat (tmp," ");
    }
    strcat (tmp,"} ");
  }
  strcat (tmp,mail_fetchsubject (stream,msgno,25));
  sprintf (tmpx," (%d chars)",cache->rfc822_size);
  strcat (tmp,tmpx);
  printf ("%s\n",tmp);
  fflush (stdout);
}

/* MM status report
 * Accepts: IMAP2 stream
 */

void status (MAILSTREAM *stream)
{
  int i;
  char date[50];
  rfc822_date (date);
  if (stream->mailbox) printf ("%s\nMailbox: %s, %d messages, %d recent\n",
			       date,stream->mailbox,stream->nmsgs,
			       stream->recent);
  else printf ("%s\n%%No mailbox is open on this stream\n",date);
  if (stream->user_flags[0]) {
    printf ("Keywords:");
    for (i = 0; i <= NUSERFLAGS; ++i)
      if (stream->user_flags[i]) printf (" %s",stream->user_flags[i]);
    printf ("\n");
  }
}


/* Prompt user for input
 * Accepts: pointer to prompt message
 *          pointer to input buffer
 */

void prompt (char *msg,char *txt)
{
#ifdef macintosh
  char *fmt = "## %s \n";
#else
  char *fmt = "%s";
#endif
  printf (fmt,msg);
  gets (txt);
}

/* Interfaces to C-client */


void mm_searched (MAILSTREAM *stream,int number)
{
  selected[number-1] = T;
}


void mm_exists (MAILSTREAM *stream,int number)
{
}


void mm_expunged (MAILSTREAM *stream,int number)
{
}


void mm_notify (MAILSTREAM *stream,char *string,short errflg)
{
  mm_log (string,errflg);
}


void mm_log (char *string,short errflg)
{
  switch (errflg) {
  case NIL:
    printf ("[%s]\n",string);
    break;
  case WARN:
    printf ("%%%s\n",string);
    break;
  case ERROR:
    printf ("?%s\n",string);
    break;
  }
}


void mm_dlog (char *string)
{
  fprintf (stderr,"%s\n",string);
}


void mm_login (char *host,char *user,char *pwd,short trial)
{
  char tmp[MAILTMPLEN];
  if (curhst) fs_give ((void **) &curhst);
  curhst = (char *) fs_get (1+strlen (host));
  strcpy (curhst,host);
  sprintf (tmp,"{%s} username: ",host);
  prompt (tmp,user);
  if (curusr) fs_give ((void **) &curusr);
  curusr = (char *) fs_get (1+strlen (user));
  strcpy (curusr,user);
  prompt ("password: ",pwd);
}

/* SMTP tester */

void smtptest (short debug)
{
  SMTPSTREAM *stream;
  char line[MAILTMPLEN];
  ENVELOPE *msg = mail_newenvelope ();
  BODY *body = mail_newbody ();
  sprintf (line,"%s <%s@%s>",personalname,curusr,curhst);
  msg->from = mtp_parse_address (line,curhst);
  sprintf (line,"%s@%s",curusr,curhst);
  msg->return_path = mtp_parse_address (line,curhst);
  while (!msg->to) {
    prompt ("To: ",line);
    msg->to = mtp_parse_address (line,curhst);
  }
  prompt ("cc: ",line);
  msg->cc = mtp_parse_address (line,curhst);
  prompt ("Subject: ",line);
  msg->subject = (char *) fs_get (1+strlen (line));
  strcpy (msg->subject,line);
  printf (" Msg (end with a line with only a '.'):\n");
  body->type = ENCTEXT;
  body->text = (char *) fs_get (8*MAILTMPLEN);
  body->text[0] = '\0';
  while (gets (line)) {
    if (line[0] == '.') {
      if (line[1] == '\0') break;
      else strcat (body->text,".");
    }
    strcat (body->text,line);
    strcat (body->text,"\015\012");
  }
  rfc822_date (line);
  msg->date = (char *) fs_get (1+strlen (line));
  strcpy (msg->date,line);
  printf ("Sending...");
  fflush (stdout);
  stream = mtp_open (hostlist,debug);
  if (stream) {
    if (mtp_mail (stream,"MAIL",msg,body)) printf ("[Ok]\n");
    else printf ("[Failed - %s]\n",stream->reply);
    mtp_close (stream);
  }
  else printf ("[Can't open connection to any server]\n");
  mail_free_envelope (&msg);
  mail_free_body (&body);
}
