/*
 * Program:	Operating-system dependent routines - Unix 4.3 version
 *
 * Author(s):	Mark Crispin
 *		Symbolic Systems Resources Group
 *		Knowledge Systems Laboratory
 *		Departments of Computer Science and Medicine
 *		Stanford University
 *		Stanford, CA  94305
 *
 * Date:	1 August 1988
 *
 * Sponsorship:	This work was supported by the Biomedical Research
 *		Technology Program of the National Institutes of Health
 *		under grant number RR-00785.
 *
 * Copyright 1988, 1989, 1990, 1991 by The Leland Stanford Junior University.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license from the Stanford Office of
 * Technology Licensing.
 *
 */


#ifdef __STDC__
#include <stdlib.h>
#else
#include "ansi.h"
#endif
#include <sys/types.h>
#include <sys/time.h>
#include <syslog.h>
#include "mail.h"
#include "osdep.h"
extern char *timezone ();

/* Write current time in RFC 822 format
 * Accepts: destination string
 */

char *days[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
char *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

void rfc822_date (char *date)
{
  int zone;
  char *zonename;
  struct tm *t;
  struct timeval tv;
  struct timezone tz;
  gettimeofday (&tv,&tz);	/* get time and timezone poop */
  t = localtime (&tv.tv_sec);	/* convert to individual items */
#if T
  zone = t->tm_gmtoff/60;	/* use this if TZ environment stuff present */
  zonename = t->tm_zone;
#else
				/* use this for older systems */
  zone = (t->tm_isdst ? 60 : 0) -tz.tz_minuteswest;
  zonename = timezone (tz.tz_minuteswest,t->tm_isdst);
#endif
				/* and output it */
  sprintf (date,"%s, %d %s %d %d:%02d:%02d %+03d%02d (%s)",
	   days[t->tm_wday],t->tm_mday,months[t->tm_mon],t->tm_year+1900,
	   t->tm_hour,t->tm_min,t->tm_sec,zone/60,abs (zone) % 60,zonename);
}

/* Get a block of free storage
 * Accepts: size of desired block
 * Returns: free storage block
 */

void *fs_get (size_t size)
{
  void *block = malloc (size);
  if (!block) fatal ("Out of free storage");
  return (block);
}


/* Resize a block of free storage
 * Accepts: ** pointer to current block
 *	    new size
 */

void fs_resize (void **block,size_t size)
{
  if (!(*block = realloc (*block,size))) fatal ("Can't resize free storage");
}


/* Return a block of free storage
 * Accepts: ** pointer to free storage block
 */

void fs_give (void **block)
{
  free (*block);
  *block = NIL;
}


/* Report a fatal error
 * Accepts: string to output
 */

void fatal (char *string)
{
  mm_log (string,ERROR);	/* output the string */
  syslog (LOG_ALERT,"IMAP C-Client crash: %s",string);
  abort ();			/* die horribly */
}

/* Copy string with CRLF newlines
 * Accepts: destination string
 *	    pointer to size of destination string
 *	    source string
 *	    length of source string
 */

char *strcrlfcpy (char **dst,long *dstl,char *src,long srcl)
{
  long i,j;
  char *d = src;
				/* count number of LF's in source string(s) */
  for (i = srcl,j = 0; j < srcl; j++) if (*d++ == '\012') i++;
				/* resize if not enough space */
  if (i > *dstl) fs_resize ((void **) dst,(*dstl = i) + 1);
  d = *dst;			/* destination string */
  while (srcl--) {		/* copy strings, inserting CR's before LF's */
    switch (*src) {
    case '\015':		/* unlikely carriage return */
      *d++ = *src++;		/* copy it and any succeeding linefeed */
      if (*src == '\012') *d++ = *src++;
      break;
    case '\012':		/* line feed? */
      *d++ ='\015';		/* yes, prepend a CR, drop into default case */
    default:			/* ordinary chararacter */
      *d++ = *src++;		/* just copy character */
      break;
    }
  }
  *d = '\0';			/* tie off destination */
  return *dst;			/* return destination */
}
