/*
 * Program:	Simple Mail Transfer Protocol (SMTP) definitions
 *
 * Author(s):	Mark Crispin
 *		Symbolic Systems Resources Group
 *		Knowledge Systems Laboratory
 *		Departments of Computer Science and Medicine
 *		Stanford University
 *		Stanford, CA  94305
 *
 * Date:	27 July 1988
 *
 * Sponsorship:	This work was supported by the Biomedical Research
 *		Technology Program of the National Institutes of Health
 *		under grant number RR-00785.
 *
 * Copyright 1988, 1989, 1990 by The Leland Stanford Junior University.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license from the Stanford Office of
 * Technology Licensing.
 *
 */


/* Constants */

#define SMTPTCPPORT 25		/* assigned TCP contact port */
#define SMTPGREET 220		/* SMTP successful greeting */
#define SMTPOK 250		/* SMTP OK code */
#define SMTPREADY 354		/* SMTP ready for data */
#define SMTPSOFTFATAL 421	/* SMTP soft fatal code */
#define SMTPHARDERROR 554	/* SMTP miscellaneous hard failure */


/* SMTP I/O stream */

typedef struct smtp_stream {
  void *tcpstream;		/* TCP I/O stream */
  short lock;			/* stream lock */
  short debug;			/* stream debug flag */
  char *reply;			/* last reply string */
} SMTPSTREAM;

/* Coddle certain compilers' 6-character symbol limitation */

#ifdef __COMPILER_KCC__
#define mtp_open xopen
#define mtp_close xclose
#define mtp_mail xmail
#define mtp_parse_address xpaddr
#define mtp_debug xdebug
#define mtp_nodebug xndbug

#define smtp_rcpt srcpt
#define smtp_address saddr
#define smtp_send ssend
#define smtp_reply sreply
#define smtp_fake sfake
#endif


/* Function prototypes */

SMTPSTREAM *mtp_open (char **hostlist,short debug);
void mtp_close (SMTPSTREAM *stream);
int mtp_mail (SMTPSTREAM *stream,char *type,ENVELOPE *msg,BODY *body);
ADDRESS *mtp_parse_address (char *string,char *defaulthost);
void mtp_debug (SMTPSTREAM *stream);
void mtp_nodebug (SMTPSTREAM *stream);
void smtp_rcpt (SMTPSTREAM *stream,ADDRESS *adr,int *error);
int smtp_send (SMTPSTREAM *stream,char *command,char *args);
int smtp_reply (SMTPSTREAM *stream);
int smtp_fake (SMTPSTREAM *stream,int code,char *text);
