/*
 * Program:	Tenex mail (TENEX) definitions
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	22 May 1990
 *
 * Copyright 1990 by University of Washington.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */

/* Build parameters */


/* Command bits from tenex_getflags(), must correspond to mailbox bit values */

#define fSEEN 1
#define fDELETED 2
#define fFLAGGED 4
#define fANSWERED 8


/* TENEX per-message cache information */

typedef struct file_cache {
  long internal;		/* pointer to internal header */
  long header;			/* pointer to RFC 822 header */
  long headersize;		/* size of RFC 822 header */
  long body;			/* pointer to message body */
  long bodysize;		/* size of message body */
  long date;			/* integer representation of date */
} FILECACHE;


/* TENEX I/O stream local data */
	
typedef struct tenex_local {
  char *host;			/* local host name */
  short inbox;			/* if this is an INBOX or not */
  int fd;			/* file descriptor for I/O */
  off_t filesize;		/* file size parsed */
  FILECACHE *msgs[MAXMESSAGES];	/* pointers to message-specific information */
  char *text;			/* mailbox text buffer */
  long textsize;		/* size of mailbox text buffer */
  long textend;			/* end of text in mailbox text buffer */
  char *buf;			/* temporary buffer */
  long buflen;			/* current size of temporary buffer */
} TENEXLOCAL;


/* Convenient access to local data */

#define LOCAL ((TENEXLOCAL *) stream->local)

/* Function prototypes */

DRIVER *tenex_valid (char *name);
MAILSTREAM *tenex_open (MAILSTREAM *stream);
void tenex_close (MAILSTREAM *stream);
void tenex_fetchfast (MAILSTREAM *stream,char *sequence);
void tenex_fetchflags (MAILSTREAM *stream,char *sequence);
ENVELOPE *tenex_fetchenvelope (MAILSTREAM *stream,int msgno);
char *tenex_fetchheader (MAILSTREAM *stream,int msgno);
char *tenex_fetchtext (MAILSTREAM *stream,int msgno);
char *tenex_fetchbody (MAILSTREAM *stream,int msgno,int section);
void tenex_setflag (MAILSTREAM *stream,char *sequence,char *flag);
void tenex_clearflag (MAILSTREAM *stream,char *sequence,char *flag);
void tenex_search (MAILSTREAM *stream,char *criteria);
MAILSTREAM *tenex_ping (MAILSTREAM *stream);
void tenex_check (MAILSTREAM *stream);
void tenex_expunge (MAILSTREAM *stream);
void tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox);
void tenex_move (MAILSTREAM *stream,char *sequence,char *mailbox);

char *tenex_file (char *dst,char *name);
short tenex_getflags (MAILSTREAM *stream,char *flag,long *uf);
short tenex_parse (MAILSTREAM *stream);
void tenex_copy_messages (MAILSTREAM *stream,unsigned char s[],char *mailbox);
MESSAGECACHE *tenex_elt (MAILSTREAM *stream,int msgno);
void tenex_update_status (MAILSTREAM *stream,int msgno);
short tenex_getsequence (MAILSTREAM *stream,unsigned char s[],char *sequence);
char tenex_search_all (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_answered (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_deleted (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_flagged (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_keyword (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_new (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_old (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_recent (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_seen (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_unanswered (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_undeleted (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_unflagged (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_unkeyword (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_unseen (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_before (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_on (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_since (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_body (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_subject (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_text (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_bcc (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_cc (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_from (MAILSTREAM *stream,int msgno,char *d,long n);
char tenex_search_to (MAILSTREAM *stream,int msgno,char *d,long n);
char (*tenex_search_date
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,long n),long *n))
     (MAILSTREAM *stream,int msgno,char *d,long n);
int tenex_date (char *s);
char (*tenex_search_flag
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,long n),long *n,
       MAILSTREAM *stream)) (MAILSTREAM *stream,int msgno,char *d,long n);
char (*tenex_search_string
      (char (*f) (MAILSTREAM *stream,int msgno,char *d,long n),char **d,
       long *n)) (MAILSTREAM *stream,int msgno,char *d,long n);

char *bezerk_snarf (MAILSTREAM *stream,int msgno,int *size);
