/*
 * Program:	IMAP2 server
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	5 November 1990
 *
 * Copyright (c) 1990, 1991 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */


/* Parameter files */

#include <stdio.h>
#include <ctype.h>
#ifdef __STDC__
#include <string.h>
#else
#include "../c-client/ansi.h"
#endif
#include <netdb.h>
#include <pwd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>
#include "mail.h"
#include "misc.h"
#include "osdep.h"


/* Autologout timer */
#define TIMEOUTTICKS 12*30
#define TICKS 5


/* Size of temporary buffers */
#define TMPLEN 8192


/* Server states */

#define LOGIN 0
#define SELECT 1
#define OPEN 2
#define LOGOUT 3

/* Global storage */

char *version = "4.0(13)";	/* version number of this server */
short state = LOGIN;		/* server state */
int mackludge = 0;		/* MacMS kludge */
int timeout = 0;		/* number of ticks until time out */
MAILSTREAM *stream = NIL;	/* mailbox stream */
int nmsgs = 0;			/* number of messages */
int recent = 0;			/* number of recent messages */
char *host = NIL;		/* local host name */
char *user = NIL;		/* user name */
char *pass = NIL;		/* password */
char *home = NIL;		/* home directory */
char *flags = NIL;		/* flag text */
char cmdbuf[TMPLEN];		/* command buffer */
char *tag;			/* tag portion of command */
char *cmd;			/* command portion of command */
char *arg;			/* pointer to current argument of command */
char *lsterr = NIL;		/* last error message from c-client */
char *response = NIL;		/* command response */
char searched[MAXMESSAGES];	/* T if message searched */


/* Response texts */

char *win = "%s OK %s completed\015\012";
char *altwin = "%s OK %s\015\012";
char *lose = "%s NO %s failed: %s\015\012";
char *misarg = "%s BAD Missing required argument to %s\015\012";
char *badarg = "%s BAD Argument given to %s when none expected\015\012";
char *notimp = "%s BAD %s not yet implemented\015\012";
char *badseq = "%s BAD Bogus sequence in %s\015\012";
char *badatt = "%s BAD Bogus attribute list in %s\015\012";
char *badlit = "%s BAD Bogus literal count in %s\015\012";
char *toobig = "* BAD Command line too long\015\012";
char *nulcmd = "* BAD Null command\015\012";
char *argrdy = "+ Ready for argument\015\012";

/* Drivers we use */

extern DRIVER bezerkdriver,tenexdriver;


/* Function prototypes */

void main (int argc,char *argv[]);
void clkint ();
char *getfile (char *s);
char *snarf (char **arg);
void fetch (char *s,char *t);
void fetch_body (int i,int k);
void fetch_body_part (int i,int k);
void fetch_envelope (int i,int k);
void fetch_encoding (int i,int k);
void changed_flags (int i,short f);
void fetch_flags (int i,int k);
void fetch_internaldate (int i,int k);
void fetch_rfc822 (int i,int k);
void fetch_rfc822_header (int i,int k);
void fetch_rfc822_size (int i,int k);
void fetch_rfc822_text (int i,int k);
void pstring (char *s);
void paddr (ADDRESS *a);
int cstring (char *s);
int caddr (ADDRESS *a);

extern char *crypt (char *key,char *salt);

/* Main program */

void main (int argc,char *argv[])
{
  int i,j,k;
  char *s,*t,*v,*w;
  struct hostent *hst;
  struct passwd *pw;
  struct itimerval timer;
  void (*f) () = NIL;
  MESSAGECACHE *elt;
  mail_link (&tenexdriver);	/* install the Tenex mail driver */
  mail_link (&bezerkdriver);	/* install the Berkeley mail driver */
  gethostname (cmdbuf,TMPLEN-1);/* get local name */
  host = cpystr ((hst = gethostbyname (cmdbuf)) ? hst->h_name : cmdbuf);
  rfc822_date (cmdbuf);		/* get date/time now */
  printf ("* OK %s IMAP2 Service %s at %s\015\012",host,version,cmdbuf);
  fflush (stdout);		/* dump output buffer */
  signal (SIGALRM,clkint);	/* prepare for clock interrupt */
  timer.it_interval.tv_sec = timer.it_value.tv_sec = TICKS;
  timer.it_interval.tv_usec = timer.it_value.tv_usec = 0;
  setitimer (ITIMER_REAL,&timer,NIL);
				/* command processing loop */
  while ((state != LOGOUT) && (timeout = TIMEOUTTICKS) &&
	 fgets (cmdbuf,TMPLEN-1,stdin)) {
    timeout = 0;		/* cancel timeout */
    fs_give ((void **) &lsterr);/* no last error */
				/* find end of line */
    if (!strchr (cmdbuf,'\012')) printf (toobig);
    else if (!(tag = strtok (cmdbuf," \015\012"))) printf (nulcmd);
    else if (!(cmd = strtok (NIL," \015\012")))
      printf ("%s BAD Missing command\015\012",tag);
    else {			/* parse command */
      response = win;		/* set default response */
      ucase (cmd);		/* canonicalize command case */
				/* snarf argument */
      arg = strtok (NIL,"\015\012");
				/* LOGOUT command always valid */
      if (!strcmp (cmd,"LOGOUT")) {
	if (state == OPEN) mail_close (stream);
	stream = NIL;
	printf ("* BYE %s IMAP2 server terminating connection\015\012",host);
	state = LOGOUT;
      }

				/* kludge for MacMS */
      else if (!strcmp (cmd,"VERSION")) {
				/* single argument */
	if (!(s = snarf (&arg))) response = misarg;
	else if (arg) response = badarg;
	else {
	  if (!((i = atoi (s)) && i > 0 && i <= 4))
	    response = "%s BAD Unknown version\015\012";
	  else if (mackludge = (i == 4))
	    printf ("* OK The MacMS kludge is enabled\015\012");
	}
      }
				/* dispatch non-NOOP depending upon state */
      else if (strcmp (cmd,"NOOP")) switch (state) {
      case LOGIN:		/* waiting to get logged in */
	if (!strcmp (cmd,"LOGIN")) {
				/* two arguments */
	  if (!((user = snarf (&arg)) && (pass = snarf (&arg))))
	    response = misarg;
	  else if (arg) response = badarg;
				/* see if username and password are OK */
	  else if ((pw = getpwnam (lcase (user))) &&
		   !strcmp (pw->pw_passwd,crypt (pass,pw->pw_passwd))) {
	    setgid (pw->pw_gid);/* all OK, login in as that user */
	    setuid (pw->pw_uid);
				/* note home directory */
	    home = cpystr (pw->pw_dir);
	    state = SELECT;	/* change to select state */
	  }
	  else response = "%s NO Bad %s user name and/or password\015\012";
	}
	else response = "%s BAD Command unrecognized/login please: %s\015\012";
	break;

      case OPEN:		/* valid only when mailbox open */
				/* fetch mailbox attributes */
	if (!strcmp (cmd,"FETCH")) {
	  if (!((s = strtok (arg," ")) && (t = strtok (NIL,"\015\012"))))
	    response = misarg;
	  else fetch (s,t);	/* do the fetch */
	}
				/* store mailbox attributes */
	else if (!strcmp (cmd,"STORE")) {
				/* must have three arguments */
	  if (!((s = strtok (arg," ")) && (t = strtok (NIL," ")) &&
		(w = strtok (NIL,"\015\012")))) response = misarg;
	  else if (!strcmp (ucase (t),"+FLAGS")) f = mail_setflag;
	  else if (!strcmp (t,"-FLAGS")) f = mail_clearflag;
	  else if (!strcmp (t,"FLAGS")) {
	    mail_clearflag (stream,s,flags);
	    f = mail_setflag;	/* gross, but rarely if ever done */
	  }
	  else response = notimp;
	  if (f) {		/* if a function was selected */
	    (*f) (stream,s,w);	/* do it */
	    fetch (s,"FLAGS");	/* get the new flags status */
	  }
	}
				/* check for new mail */
	else if (!strcmp (cmd,"CHECK")) {
				/* no arguments */
	  if (arg) response = badarg;
	  else {
	    recent = nmsgs = -1;/* make sure we get an update */
	    mail_check (stream);
	  }
	}
				/* expunge deleted messages */
	else if (!strcmp (cmd,"EXPUNGE")) {
				/* no arguments */
	  if (arg) response = badarg;
	  else {
	    recent = nmsgs = -1;/* make sure we get an update */
	    mail_expunge (stream);
	  }
	}
				/* copy message(s) */
	else if (!strcmp (cmd,"COPY")) {
	  if (!((s = snarf (&arg)) && (t = snarf (&arg)))) response = misarg;
	  else if (arg) response = badarg;
	  else mail_copy (stream,s,t);
	}

				/* search mailbox */
	else if (!strcmp (cmd,"SEARCH")) {
				/* one or more arguments required */
	  if (!arg) response = misarg;
	  else {		/* zap search vector */
	    memset (searched,NIL,stream->nmsgs);
				/* end with a literal argument? */
	    while ((*(t = arg + strlen (arg) - 1) == '}') &&
		   (s = strrchr (arg,'{')) && response == win) {
				/* get length of literal, validate */
	      if (((i = atoi (++s)) < 1) || (TMPLEN - (t++ - cmdbuf) < i + 10))
		response = badlit;
	      else {
		*t++ = '\015';	/* reappend CR/LF */
		*t++ = '\012';
		printf (argrdy);/* tell client ready for argument */
				/* copy the literal */
		while (i--) *t++ = getchar ();
				/* start a timeout */
		timeout = TIMEOUTTICKS;
				/* get new command tail */
		if (!fgets (t,TMPLEN - (t-cmdbuf) - 1,stdin)) response = lose;
		timeout = 0;	/* clear timeout */
				/* find end of line */
		if (!(s = strchr (t,'\012'))) response = toobig;
		else {
		  *s = NIL;	/* tie off line */
		  if (*--s == '\015') *s = NIL;
		}
	      }
	    }
				/* punt if error */
	    if (response != win) break;
				/* do the search */
	    mail_search (stream,arg);
	    if (response == win || response == altwin) {
				/* output search results */
	      fputs ("* SEARCH",stdout);
	      for (i = 1; i <= stream->nmsgs; ++i)
		if (searched[i-1]) printf (" %d",i);
	      fputs ("\015\012",stdout);
	    }
	  }
	}
	else			/* fall into select case */

      case SELECT:		/* valid whenever logged in */
				/* select new mailbox */
	if (!strcmp (cmd,"SELECT")) {
				/* single argument */
	  if (!(s = snarf (&arg))) response = misarg;
	  else if (arg) response = badarg;
	  else {
	    recent = nmsgs = -1;/* make sure we get an update */
	    if ((stream = mail_open (stream,s,NIL)) && (response == win)) {
				/* flush old list */
	      fs_give ((void **) &flags);
	      *s = '(';		/* start new flag list over old file name */
	      s[1] = '\0';
	      for (i = 0; i < NUSERFLAGS; i++)
		if (t = stream->user_flags[i]) strcat (strcat (s,t)," ");
				/* append system flags to list */
	      strcat (s,"\\Answered \\Flagged \\Deleted \\Seen)");
				/* output list of flags */
	      printf ("* FLAGS %s\015\012",(flags = cpystr (s)));
	      state = OPEN;
	    }
	    else {
				/* nuke if still open */
	      if (stream) mail_close (stream);
	      stream = NIL;
	      state = SELECT;	/* no mailbox open now */
	      response = lose;	/* open failed */
	    }
	  }
	}
				/* select bulletin board */
	else if (!strcmp (cmd,"BBOARD")) response = notimp;

				/* find mailboxes or bboards */
	else if (!strcmp (cmd,"FIND")) {
				/* get subcommand and true argument */
	  if (!((s = strtok (arg," \015\012")) && (cmd = ucase (s)) &&
		(arg = strtok (NIL,"\015\012")) && (s = snarf (&arg))))
	    response = misarg;	/* missing required argument */
	  else if (arg) response = badarg;
	  else {		/* temporarily only allow full wild */
	    if (strcmp (s,"*")) response = "%s BAD Only `*' supported\015\012";
				/* find mailboxes */
	    else if (!strcmp (cmd,"MAILBOXES")) {
				/* get list of mailboxes */
	      if (w = getfile (".mailboxlist")) {
				/* get first mailbox name */
		if (t = strtok (w,"\n")) do
				/* output mailbox if not IMAP or INBOX */
		  if ((*t != '{') && strcmp (t,"INBOX"))
		    printf ("* MAILBOX %s\015\012",t);
				/* for each mailbox */
		while (t = strtok (NIL,"\n"));
		fs_give ((void **) &w);
	      }
	    }
				/* find bulletin boards */
	    else if (!strcmp (cmd,"BBOARDS")) {
				/* get list of bboards */
	      if (w = getfile (".newsrc")) {
				/* get first bboard name */
		if (t = strtok (w,"\n")) do if (v = strchr (t,':')) {
		  *v = '\0';	/* tie off at end of name */
		  printf ("* BBOARD %s\015\012",t);
		} while (t = strtok (NIL,"\n"));
		fs_give ((void **) &w);
	      }
	    }
	    else response = "%s BAD Command unrecognized: FIND %s\015\012";
	  }
	}

	else response = "%s BAD Command unrecognized: %s\015\012";
	break;
      default:
        response = "%s BAD Server in unknown state for %s command\015\012";
	break;
      }
      if (state == OPEN) {	/* ping open mailbox */
	if (stream = mail_ping (stream)) {
				/* change in recent messages? */
	  if (recent != stream->recent)
	    printf ("* %d RECENT\015\012",(recent = stream->recent));
	}
	else {			/* horrible death */
	  printf ("* BYE %s Fatal mailbox error\015\012",host);
	  state = LOGOUT;	/* go away */
	}
      }
				/* get text for alternative win message now */
      if (response == altwin) cmd = lsterr;
				/* output response */
      printf (response,tag,cmd,lsterr);
    }
    fflush (stdout);		/* make sure output blatted */
  }
  exit (0);			/* all done */
}


/* Clock interrupt
 */

void clkint ()
{
  if (!--timeout) {		/* did timeout run out? */
    printf ("* BYE Autologout; idle for too long\015\012");
    fflush (stdout);		/* make sure output blatted */
				/* try to gracefully close the stream */
    if (state == OPEN) mail_close (stream);
    stream = NIL;
    exit (0);			/* die die die */
  }
}

/* Get a file
 * Accepts: name of file on home directory
 * Returns: file text
 */

char *getfile (char *s)
{
  char tmp[TMPLEN];
  int fd;
  struct stat sbuf;
				/* sanity check */
  if (strlen (s) > TMPLEN/2) return NIL;
  sprintf (tmp,"%s/%s",home,s);	/* open file on home directory */
  if ((fd = open (tmp,O_RDONLY,NIL)) < 0) return NIL;
  fstat (fd,&sbuf);		/* get file size and read data */
  read (fd,s = (char *) fs_get (sbuf.st_size + 1),sbuf.st_size);
  close (fd);			/* close file */
  s[sbuf.st_size] = '\0';	/* tie off string */
  return s;
}

/* Snarf an argument
 * Accepts: pointer to argument text pointer
 * Returns: argument
 */

char *snarf (char **arg)
{
  int i;
  char *c = *arg;
  char *t = NIL;
  if (!c) return NIL;		/* better be an argument */
  switch (*c) {			/* see what the argument is */
  case '\0':			/* catch bogons */
  case ' ':
    return NIL;
  case '"':			/* quoted string */
    if (!(strchr (c+1,'"') && (c = strtok (c,"\"")))) return NIL;
    break;
  case '{':			/* literal string */
    i = strtol (c+1,&c,10);	/* get its length */
				/* validate end of literal */
    if (*c++ != '}' || *c++) return NIL;
    if ((i + 10) > (TMPLEN - (c - cmdbuf))) return NIL;
    printf (argrdy);		/* tell client ready for argument */
    t = c;			/* get it */
    while (i--) *t++ = getchar ();
    *t++ = NIL;			/* make sure string tied off */
    timeout = TIMEOUTTICKS;	/* start timeout */
    				/* get new command tail */
    if (!(fgets ((*arg = t),TMPLEN - (t - cmdbuf) - 1,stdin) &&
	  strchr (t,'\012'))) {
      timeout = 0;		/* clear timeout */
      response = toobig;	/* lost it seems */
      return NIL;
    }
    timeout = 0;		/* clear timeout */
    break;
  default:			/* atomic string */
    c = strtok (*arg," \015\012");
    break;
  }
				/* remainder of arguments */
  if ((*arg = strtok (t,"\015\012")) && **arg == ' ') ++*arg;
  return c;
}

/* Fetch message data
 * Accepts: sequence as a string
 *	    string of data items to be fetched
 */

#define MAXFETCH 100

void fetch (char *s,char *t)
{
  char c,*v;
  int i,j,k;
  void (*f[MAXFETCH]) ();
  int fa[MAXFETCH];
				/* parse sequence vector */
  memset (searched,NIL,stream->nmsgs);
  while (*s && isdigit (*s) && (response == win)) {
				/* parse and validate number */
    if (((i = strtol (s,&s,10)) < 1) || (i > stream->nmsgs)) response = badseq;
				/* delimiter indicate single sequence? */
    else if (((c = *s++) == ',') || !c) searched[i-1] = T;
				/* multiple sequence delimiter? */
    else if ((c == ':') && (((j = strtol (s,&s,10)) > 0) ||
			    (j <= stream->nmsgs)) &&
	     ((!*s) || (*s++ == ','))) {
      if (i > j) {		/* swap sequence if necessary */
	k = i; i = j; j = k;
      }
      for (k = i; k <= j; k++) searched[k-1] = T;
    }
    else response = badseq;
  }
  if (response != win) return;	/* punt now if failed */
				/* process macros */
  if (!strcmp (ucase (t),"(ALL ENCODING)"))
    strcpy (t,"(FLAGS INTERNALDATE RFC822.SIZE ENVELOPE ENCODING)");
  else if (!strcmp (t,"ALL"))
    strcpy (t,"(FLAGS INTERNALDATE RFC822.SIZE ENVELOPE)");
  else if (!strcmp (t,"FAST")) strcpy (t,"(FLAGS INTERNALDATE RFC822.SIZE)");
  s = t + strlen (t) - 1;	/* last character in attribute string */
				/* if multiple items, make sure in list form */
  if (strchr (t,' ') && ((*t != '(') || (*s != ')'))) {
    response = badatt;
    return;
  }
				/* nuke the parens now */
  if ((*t == '(') && (*s == ')') && t++) *s = '\0';

  k = 0;			/* initial index */
  if (s = strtok (t," ")) do {	/* parse attribute list */
    if (*s == 'B' && s[1] == 'O' && s[2] == 'D' && s[3] == 'Y') switch (s[4]) {
    case '\0':			/* entire body */
      f[k++] = fetch_body;
      break;
    case '[':			/* body segment */
      if ((v = strchr (s,']')) && (!(*v = v[1])) &&
	  ((fa[k] = strtol (s+5,&s,10)) >= 0)) {
	f[k++] = fetch_body_part;
	break;			/* valid body segment */
      }				/* else fall into default case */
    default:			/* bogus */
      response = badatt;
      return;
    }
    else if (!strcmp (s,"ENVELOPE")) f[k++] = fetch_envelope;
    else if (!strcmp (s,"ENCODING")) f[k++] = fetch_encoding;
    else if (!strcmp (s,"FLAGS")) f[k++] = fetch_flags;
    else if (!strcmp (s,"INTERNALDATE")) f[k++] = fetch_internaldate;
    else if (!strcmp (s,"RFC822")) f[k++] = fetch_rfc822;
    else if (!strcmp (s,"RFC822.HEADER")) f[k++] = fetch_rfc822_header;
    else if (!strcmp (s,"RFC822.SIZE")) f[k++] = fetch_rfc822_size;
    else if (!strcmp (s,"RFC822.TEXT")) f[k++] = fetch_rfc822_text;
    else {			/* unknown attribute */
      response = badatt;
      return;
    }
  } while ((s = strtok (NIL," ")) && k < MAXFETCH);
  else {
    response = misarg;		/* missing attribute list */
    return;
  }
  if (s) {			/* too many attributes? */
    response = "%s BAD Excessively complex FETCH attribute list\015\012";
    return;
  }
  f[k++] = NIL;			/* tie off attribute list */
				/* for each requested message */
  for (i = 1;i <= stream->nmsgs; i++) if (searched[i-1]) {
				/* parse envelope, set body, do warnings */
    mail_fetchenvelope (stream,i);
    printf ("* %d FETCH (",i);	/* leader */
    (*f[0]) (i,fa[0]);		/* do first attribute */
    for (k = 1; *f[k]; k++) {	/* for each subsequent attribute */
      putchar (' ');		/* delimit with space */
      (*f[k]) (i,fa[k]);	/* do that attribute */
    }
    fputs (")\015\012",stdout);	/* trailer */
  }
}

/* Fetch RFC-1154 structured message body
 * Accepts: message number
 *	    argument
 */

BODY fakebody = {0,ENCTEXT,NIL,NIL,NIL};

void fetch_body (int i,int k)
{
  MESSAGECACHE *elt = mail_elt (stream,i);
  short f = elt->seen;
  BODY *body = elt->body ? elt->body : &fakebody;
  fputs ("BODY (",stdout);	/* output attribute */
  k = 0;			/* start with section 0 */
  do {				/* output each body section */
    printf (body->options ? "(%d %s %s) " : "(%d %s) ",body->count,
	    ent[body->type],body->options);
    pstring (mail_fetchbody (stream,i,k++));
    if (body->next) putchar (' ');
  } while (body = body->next);	/* until no more sections */
  putchar (')');		/* close body list */
  changed_flags (i,f);		/* output flags */
}


/* Fetch RFC-1154 message encoding
 * Accepts: message number
 *	    argument
 */

void fetch_encoding (int i,int k)
{
  MESSAGECACHE *elt = mail_elt (stream,i);
  BODY *body = elt->body;
  fputs ("ENCODING ",stdout);	/* output attribute */
  if (body) {			/* have a message body? */
    putchar ('(');		/* open encoding list */
    do printf (body->options ? "(%d %s %s)" : "(%d %s)",body->count,
	       ent[body->type],body->options);
    while (body = body->next);
    putchar (')');		/* close encoding list */
  }
  else fputs ("NIL",stdout);
}

/* Fetch RFC-1154 body part
 * Accepts: message number
 *	    argument
 */

void fetch_body_part (int i,int k)
{
  int j = k;
  MESSAGECACHE *elt = mail_elt (stream,i);
  short f = elt->seen;
  BODY *body = elt->body;
  printf ("BODY[%d] ",k);	/* output attribute */
				/* search for section if have a body */
  if (body) while (j-- && (body = body->next));
  else if (!k) body = &fakebody;/* else body is a single text segment */
  if (body) {			/* have a body to output? */
    printf (body->options ? "((%d %s %s) " : "((%d %s) ",body->count,
	    ent[body->type],body->options);
    pstring (mail_fetchbody (stream,i,k));
    putchar (')');		/* close off structure */
    changed_flags (i,f);	/* output changed flags */
  }
  else fputs ("NIL",stdout);	/* can't use anything at all */
}

/* Fetch IMAP envelope
 * Accepts: message number
 *	    argument
 */

void fetch_envelope (int i,int k)
{
  ENVELOPE *env = mail_fetchenvelope (stream,i);
  fputs ("ENVELOPE ",stdout);	/* output attribute */
  if (env) {			/* have an envelope? */
				/* disgusting MacMS kludge */
    if (mackludge) printf ("%d ",cstring (env->date) + cstring (env->subject) +
			   caddr (env->from) + caddr (env->sender) +
			   caddr (env->reply_to) + caddr (env->to) +
			   caddr (env->cc) + caddr (env->bcc) +
			   cstring (env->in_reply_to) +
			   cstring (env->message_id));
    putchar ('(');		/* delimiter */
    pstring (env->date);	/* output envelope fields */
    putchar (' ');
    pstring (env->subject);
    putchar (' ');
    paddr (env->from);
    putchar (' ');
    paddr (env->sender);
    putchar (' ');
    paddr (env->reply_to);
    putchar (' ');
    paddr (env->to);
    putchar (' ');
    paddr (env->cc);
    putchar (' ');
    paddr (env->bcc);
    putchar (' ');
    pstring (env->in_reply_to);
    putchar (' ');
    pstring (env->message_id);
    putchar (')');		/* end of envelope */
  }
  else fputs ("NIL",stdout);	/* no envelope */
}

/* Fetch flags
 * Accepts: message number
 *	    argument
 */

void fetch_flags (int i,int k)
{
  char tmp[TMPLEN];
  long u;
  char *s = tmp;
  char *t;
  MESSAGECACHE *elt = mail_elt (stream,i);
  s[0] = s[1] = '\0';		/* start with empty flag string */
				/* output system flags */
  if (elt->recent) strcat (s," \\Recent");
  if (elt->seen) strcat (s," \\Seen");
  if (elt->deleted) strcat (s," \\Deleted");
  if (elt->flagged) strcat (s," \\Flagged");
  if (elt->answered) strcat (s," \\Answered");
  if (u = elt->user_flags) do	/* any user flags? */
    if ((TMPLEN - ((s += strlen (s)) - tmp)) >
	(2 + strlen (t = stream->user_flags[find_rightmost_bit (&u)]))) {
      *s++ = ' ';		/* space delimiter */
      strcpy (s,t);		/* copy the user flag */
    }
  while (u);			/* until no more user flags */
  printf ("FLAGS (%s)",tmp+1);	/* output results, skip first char of list */
}


/* Output flags if was unseen
 * Accepts: message number
 *	    prior value of Seen flag
 */

void changed_flags (int i,short f)
{
  if (!f) {			/* was unseen? */
    putchar (' ');		/* yes, delimit with space */
    fetch_flags (i,i);		/* output flags */
  }
}


/* Fetch message internal date
 * Accepts: message number
 *	    argument
 */

void fetch_internaldate (int i,int k)
{
  printf ("INTERNALDATE \"%s\"",mail_elt (stream,i)->internal_date);
}

/* Fetch complete RFC-822 format message
 * Accepts: message number
 *	    argument
 */

void fetch_rfc822 (int i,int k)
{
  short f = mail_elt (stream,i)->seen;
  printf ("RFC822 {%d}\015\012%s",mail_elt (stream,i)->rfc822_size,
	  mail_fetchheader (stream,i));
  fputs (mail_fetchtext (stream,i),stdout);
  changed_flags (i,f);		/* output changed flags */
}


/* Fetch RFC-822 header
 * Accepts: message number
 *	    argument
 */

void fetch_rfc822_header (int i,int k)
{
  fputs ("RFC822.HEADER ",stdout);
  pstring (mail_fetchheader (stream,i));
}


/* Fetch RFC-822 message length
 * Accepts: message number
 *	    argument
 */

void fetch_rfc822_size (int i,int k)
{
  printf ("RFC822.SIZE %d",mail_elt (stream,i)->rfc822_size);
}


/* Fetch RFC-822 text only
 * Accepts: message number
 *	    argument
 */

void fetch_rfc822_text (int i,int k)
{
  short f = mail_elt (stream,i)->seen;
  fputs ("RFC822.TEXT ",stdout);
  pstring (mail_fetchtext (stream,i));
  changed_flags (i,f);		/* output changed flags */
}

/* Print string
 * Accepts: string
 */

void pstring (char *s)
{
  char c,*t;
  if (t = s) {			/* is there a string? */
				/* yes, search for specials */
    while (c = *t++) if (c == '"' || c == '{' || c == '\015' || c == '\012' ||
			 c == '%' || c == '\\') break;
				/* must use literal string */
    if (c) printf ("{%d}\015\012%s",strlen (s),s);
    else printf ("\"%s\"",s);	/* may use quoted string */
  }
  else fputs ("NIL",stdout);	/* empty string */
}


/* Print address list
 * Accepts: address list
 */

void paddr (ADDRESS *a)
{
  if (a) {			/* have anything in address? */
    putchar ('(');		/* open the address list */
    do {			/* for each address */
      putchar ('(');		/* open the address */
      pstring (a->personal);	/* personal name */
      putchar (' ');
      pstring (a->route_list);	/* at-domain-list */
      putchar (' ');
      pstring (a->mail_box);	/* mailbox */
      putchar (' ');
      pstring (a->host);	/* domain name of mailbox's host */
      putchar (')');		/* terminate address */
    } while (a = a->next);	/* until end of address */
    putchar (')');		/* close address list */
  }
  else fputs ("NIL",stdout);	/* empty address */
}

/* Count string and space afterwards
 * Accepts: string
 */

int cstring (char *s)
{
  char tmp[20];
  char c,*t;
  int i;
  if (t = s) {			/* is there a string? */
				/* yes, search for specials */
    while (c = *t++) if (c == '"' || c == '{' || c == '\015' || c == '\012' ||
			 c == '%' || c == '\\') break;
    if (c) {			/* must use literal string */
      sprintf (tmp,"{%d}\015\012",i = strlen (s));
      return strlen (tmp) + i + 1;
    }
    else return 3 + strlen (s);	/* quoted string */
  }
  else return 4;		/* NIL plus space */
}


/* Count address list and space afterwards
 * Accepts: address list
 */

int caddr (ADDRESS *a)
{
  int i = 3;			/* open, close, and space */
				/* count strings in address list */
  if (a) do i += 1 + cstring (a->personal) + cstring (a->route_list) +
    cstring (a->mail_box) + cstring (a->host);
  while (a = a->next);		/* until end of address */
  else i = 4;			/* NIL plus space */
  return i;			/* return the count */
}

/* Co-routines from MAIL library */


/* Message matches a search
 * Accepts: IMAP2 stream
 *	    message number
 */

void mm_searched (MAILSTREAM *stream,int msgno)
{
  if (msgno > 0 && msgno <= MAXMESSAGES)
    searched[msgno - 1] = T;	/* note message selected */
}


/* Message exists (i.e. there are that many messages in the mailbox)
 * Accepts: IMAP2 stream
 *	    message number
 */

void mm_exists (MAILSTREAM *stream,int number)
{
				/* note change in number of messages */
  if (nmsgs != number) printf ("* %d EXISTS\015\012",(nmsgs = number));
}


/* Message expunged
 * Accepts: IMAP2 stream
 *	    message number
 */

void mm_expunged (MAILSTREAM *stream,int number)
{
  printf ("* %d EXPUNGE\015\012",number);
}

/* Notification event
 * Accepts: IMAP2 stream
 *	    string to log
 *	    error flag
 */

void mm_notify (MAILSTREAM *stream,char *string,short errflg)
{
  mm_log (string,errflg);	/* just do mm_log action */
}


/* Log an event for the user to see
 * Accepts: string to log
 *	    error flag
 */

void mm_log (char *string,short errflg)
{
  switch (errflg) {		/* action depends upon the error flag */
  case NIL:			/* information message, set as OK response */
    if (response == win) {	/* only if no other response yet */
      response = altwin;	/* switch to alternative win message */
      fs_give ((void **) &lsterr);
      lsterr = cpystr (string);	/* copy string for later use */
    }
    break;
  case WARN:			/* warning, output unsolicited OK notice */
    printf ("* OK %s\015\012",string);
    break;
  case ERROR:			/* error that broke command */
  default:			/* default should never happen */
    response = lose;		/* set fatality */
    fs_give ((void **) &lsterr);/* flush old error */
    lsterr = cpystr (string);	/* note last error */
    break;
  }
}


/* Log an event to debugging telemetry
 * Accepts: string to log
 */

void mm_dlog (char *string)
{
  mm_log (string,WARN);		/* shouldn't happen normally */
}

/* Get user name and password for this host
 * Accepts: host name
 *	    where to return user name
 *	    where to return password
 *	    trial count
 */

void mm_login (char *host,char *username,char *password,short trial)
{
  strcpy (username,user);	/* set user name */
  strcpy (password,pass);	/* and password */
}
