/*
 * Program:	Distributed Electronic Mail Manager (AddressBookCell object)
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	22 March 1989
 *
 * Copyright (c) 1990 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */


#import "MailManager.h"

@implementation AddressBookCell

// Create a new AddressBookCell

+ new
{
  self = [super newTextCell:"(*) *"];
  [self setFont:defaultfont];	// set up default font and left-aligned
  [self setAlignment:NX_LEFTALIGNED];
  nickName = fullName = address = extra = NIL;
  return self;
}


// Clean up

- free
{
				// free up strings
  if (nickName) fs_give ((void **) &nickName);
  if (fullName) fs_give ((void **) &fullName);
  if (address) fs_give ((void **) &address);
  if (extra) fs_give ((void **) &extra);
  return [super free];
}

// Set its fields

- setNickName:(char *) newNickName fullName:(char *) newFullName
 address:(char *) newAddress extra:(char *) newExtra;
{
  char tmp[TMPLEN];
				// copy the arguments
  nickName = (const char *) cpystr (newNickName);
  fullName = (const char *) cpystr (newFullName);
  address = (const char *) cpystr (newAddress);
  extra = (const char *) cpystr (newExtra);
				// generate title
  sprintf (tmp,"(%s) %s",nickName,fullName);
  [self setStringValue:(const char *) tmp];
  return self;
}


// Return nickname field

- (const char *) nickName
{
  return nickName;
}


// Return name field

- (const char *) fullName
{
  return fullName;
}


// Return address field

- (const char *) address
{
  return address;
}


// Return extra field

- (const char *) extra
{
  return extra;
}


@end
