/*
 * Program:	Distributed Electronic Mail Manager (BrowserCell object)
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	30 March 1989
 *
 * Copyright (c) 1989, 1990 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */


#import "MailManager.h"

@implementation BrowserCell

// Create a new BrowserCell

+ new
{
  char hdr[256];
  headerline (hdr,NIL,NIL);	// make the prototype header line
  self = [super newTextCell:hdr];
  [self setFont:defaultfont];	// set up default font and left-aligned
  [self setAlignment:NX_LEFTALIGNED];
  stream = NIL; elt = NIL;	// initialize stream and elt
  return self;
}


// Note the stream and the element that owns us

- setStream:(MAILSTREAM *) s element:(MESSAGECACHE *) e
{
  stream = s; elt = e;		// note the stream and element
  return self;
}

// Draw the contents of the BrowserCell

- drawInside:(NXRect *) cellFrame inView:(Matrix *) controlView
{
				// set string value to match current header
  if (stream && !mail_checklock (stream))
    headerline ((char *) [self stringValue],stream,elt);
				// is message flagged?
  if (elt && elt->flagged) {
    NXRect frame;
    int sides[] = {NX_YMIN, NX_XMAX, NX_YMAX, NX_XMIN};
    float grays[] = {NX_BLACK, NX_BLACK, NX_BLACK, NX_BLACK};
				// copy frame since NXDrawTiledRects clobbers
    NXSetRect (&frame,NX_X (cellFrame),NX_Y (cellFrame),
	       NX_WIDTH (cellFrame),NX_HEIGHT (cellFrame));
				// draw a black rectangle on all four sides
    NXDrawTiledRects (&frame,(NXRect *) NIL,sides,grays,4);
  }
				// now draw the current header
  [super drawInside:cellFrame inView:controlView];
  return self;
}


// Set the attributes of the Text object used for display

- setTextAttributes:(Text *) textObj
{
				// do the superior's action first
  [super setTextAttributes:textObj];
				// make text dark gray if deleted, else black
  [textObj setTextGray:((elt && elt->deleted) ? NX_DKGRAY : NX_BLACK)];
  return textObj;
}


// Return the message number for this BrowserCell

- (int) msgno
{
				// return the message number
  return (elt ? elt->msgno : NIL);
}


@end
