/*
 * Program:	Distributed Electronic Mail Manager (MailManager object)
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	17 February 1989
 *
 * Copyright (c) 1989, 1990, 1991 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */

// Preliminary definitions

#define TMPLEN 1024
#define STREAMPROP struct stream_prop
#define SEQUENCE struct msg_sequence


// Everything we snarf from elsewhere

#import <appkit/appkit.h>
#import "../c-client/mail.h"
#import "../c-client/smtp.h"
#import "../c-client/rfc822.h"
#import "../c-client/misc.h"
#import "../c-client/osdep.h"
#import "ReadWindow.h"
#import "SendWindow.h"
#import "MBoxWindow.h"
#import "AddressBook.h"
#import "AddressBookCell.h"
#import "AttachedPanel.h"
#import "BrowserCell.h"
#import "PullOutMenu.h"
#import "ReplyWindow.h"
#import "Utilities.h"
#import <netdb.h>
#import <pwd.h>
#import <time.h>
#import <sys/types.h>
#import <sys/stat.h>
#import <sys/time.h>
#import <sys/timeb.h>

// Linkages to MAIL drivers

extern DRIVER imapdriver,bezerkdriver,tenexdriver;


// Prototypes of functions certain individuals would rather we not use

void ftime (struct timeb *tb);
char *timezone (int zone,int dst);
char *mktemp (char *template);


// Mailbox formats

#define MBOX 0			// Bezerkley format
#define MTXT 1			// Tenex format


// Alias global variable names

#define mtpserver hostlist[0]
#define domainname hostlist[1]


// Global variables

extern const char *defaultbcc,*defaultcc,*outbox;
extern const char *personalname,*repository,*username;
extern BOOL autoexpunge,autologin,autoopen,autoread,autozoom;
extern BOOL closefinalkill,literaldisplay,nolinebreaking,nopersonal;
extern BOOL lockedopen,lockedread;
extern short debug,format;
extern double interval;
extern char *localhost,*localuser,*localpersonal,*lastlogin;
extern Font *defaultfont;
extern const char *hostlist[],*months[];

@interface MailManager : Application
{
  id telemetryView;		// view to draw telemetry in
  id prefDomainName;		// domain name (for sending)
  id prefMTPServer;		// preferred SMTP server
  id prefPersonalName;		// personal name (for sending)
  id prefRepository;		// preferred repository
  id prefUserName;		// preferred user name (for sending)
  id defaultBccList;		// default bcc list (for sending)
  id defaultCcList;		// default cc list (for sending)
  id outBox;			// saved messages file
  id mailboxFormat;		// mailbox format
  id automaticExpunge;		// automatic expunge when mailbox closed
  id automaticLogin;		// automatic login with previous password
  id automaticRead;		// automatic read when open
  id automaticOpen;		// automatic open of default mailbox
  id automaticZoom;		// automatic zoom on open
  id closeFinalKill;		// close read window on final kill
  id debugging;			// default debugging state
  id literalDisplay;		// default display state (filtered vs. literal)
  id noLineBreaking;		// suppress send window line breaking
  id noPersonalNames;		// suppress personal names in recipient lists
  id checkInterval;		// interval between implicit checks
  id mailboxSelection;		// mailbox selection panel
  id mailboxes;			// view of mailbox browser
  id browser;			// matrix of mailbox browser
  id mailboxResponder;		// new mailbox responder
  id mailboxHost;		// new mailbox host name
  id mailboxMailbox;		// new mailbox mailbox name
  id login;			// login panel
  id loginBox;			// box that encloses login responder
  id loginResponder;		// login responder
  id password;			// login password
  id userName;			// login user name
  id printer;			// printer view
  id opener;			// created mailbox window
  id fileDestination;		// destination file string
  id fileLocal;			// local file desired
  id filePanel;			// panel for Move/Copy commands
  id helpPanel;			// panel for menu help
  id addressHelpPanel;		// address book help panel
  id mboxHelpPanel;		// mailbox help panel
  id selectHelpPanel;		// select help panel
  id readHelpPanel;		// read help panel
  id sendHelpPanel;		// send help panel
  id infoPanel;			// panel for program ID
  id preferencesPanel;		// panel for preferences
  id preferencesHelp;		// panel for preferences help
  id book;			// address book
  id pasteboard;		// pasteboard
  BOOL OK;			// OK flag
}

+ new;
- run;
- compose:sender;
- open:sender;
- reOpen:(MAILSTREAM *) stream window:old;
- close;
- addMailboxToBrowser:(char *) name;
- setMailboxSelection:anObject;
- setMailboxes:anObject;
- setMailboxResponder:anObject;
- setMailboxHost:anObject;
- setMailboxMailbox:anObject;
- selectLoad:sender;
- mailboxCancel:sender;
- mailboxOK:sender;
- setLogin:anObject;
- setLoginBox:anObject;
- setLoginResponder:anObject;
- setUserName:anObject;
- setPassword:anObject;
- login:(char *) host user:(char *) user password:(char *) pwd force:(BOOL) f;
- loginOK:sender;
- setTelemetryView:anObject;
- telemetry:(char *) string;
- setPrinter:anObject;
- startPrint;
- printText:(char *) text;
- print;
- define:sender;
- setFilePanel:anObject;
- setFileDestination:anObject;
- setFileLocal:anObject;
- (char *) getFile;
- fileCancel:sender;
- fileOK:sender;
- setHelpPanel:anObject;
- help:sender;
- setAddressHelpPanel:anObject;
- addressHelp:sender;
- setMboxHelpPanel:anObject;
- mboxHelp:sender;
- setSelectHelpPanel:anObject;
- selectHelp:sender;
- setReadHelpPanel:anObject;
- readHelp:sender;
- setSendHelpPanel:anObject;
- sendHelp:sender;
- setInfoPanel:anObject;
- info:sender;

- setPreferencesPanel:anObject;
- setPreferencesHelp:anObject;
- setPrefDomainName:anObject;
- setPrefMTPServer:anObject;
- setPrefPersonalName:anObject;
- setPrefRepository:anObject;
- setPrefUserName:anObject;
- setDefaultBccList:anObject;
- setDefaultCcList:anObject;
- setOutBox:anObject;
- setMailboxFormat:anObject;
- setAutomaticExpunge:anObject;
- setAutomaticLogin:anObject;
- setAutomaticRead:anObject;
- setAutomaticOpen:anObject;
- setAutomaticZoom:anObject;
- setCloseFinalKill:anObject;
- setDebugging:anObject;
- setLiteralDisplay:anObject;
- setNoLineBreaking:anObject;
- setNoPersonalNames:anObject;
- setCheckInterval:anObject;
- setPreferencesHelp:anObject;
- domainName:sender;
- mtpServer:sender;
- personalName:sender;
- repository:sender;
- userName:sender;
- defaultBccList:sender;
- defaultCcList:sender;
- outBox:sender;
- mailboxFormat:sender;
- automaticExpunge:sender;
- automaticLogin:sender;
- automaticRead:sender;
- automaticOpen:sender;
- automaticZoom:sender;
- closeFinalKill:sender;
- debug:sender;
- literalDisplay:sender;
- noLineBreaking:sender;
- noPersonalNames:sender;
- checkInterval:sender;
- preferences:sender;
- helpPreferences:sender;
- savePreferences:sender;
- addressBook:sender;
- setPasteboard:(char *) text;
- terminate:sender;

@end
