/*
 * Program:	Distributed Electronic Mail Manager (ReplyWindow object)
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	27 June 1989
 *
 * Copyright (c) 1989, 1990, 1991 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */


#import "MailManager.h"

@implementation ReplyWindow

// Create a new ReplyWindow

+ new:(MAILHANDLE *) h elt:(MESSAGECACHE *) e text:(char *) text all:(BOOL) all
{
  char tmp[16*TMPLEN],frm[TMPLEN],c,*src,*dst,*d;
  int i;
  ENVELOPE *msg;
  self = [super new];		// open a SendWindow, set as window's delegate
  [[self window] setDelegate:self];
  handle = h;			// note handle for later
  (elt = e)->lockcount++;	// note cache element for later and lock
				// copy reply-to
  (msg = [self msg])->to = copy_adr (e->env->reply_to,NIL);
  if (all) {			// copy to/cc lists if "reply all"?
    if (msg->to) msg->cc = copy_adr (e->env->cc,copy_adr (e->env->to,msg->cc));
    else msg->to = copy_adr (e->env->cc,copy_adr (e->env->to,NIL));
    msg->bcc = copy_adr (e->env->bcc,msg->bcc);
  }
  if (e->env->subject) {	// use subject in reply if present
    strncpy (tmp,e->env->subject,3);
    tmp[3] ='\0';		// tie off copy of first 3 chars
    ucase (tmp);		// make the whole thing uppercase
				// a "re:" already there?
    if (strcmp (tmp,"RE:")) sprintf (tmp,"re: %s",e->env->subject);
    else sprintf (tmp,"%s",e->env->subject);
  }
  else sprintf (tmp,"(response to message of %s)",e->env->date);
  msg->subject = cpystr (tmp);	// copy desired subject
				// calculate simple from
  if (!e->env->from) strcpy (frm,"(unknown)");
  else if (e->env->from->personal) strcpy (frm,e->env->from->personal);
  else sprintf (frm,"%s@%s",e->env->from->mail_box,e->env->from->host);
				// in-reply-to is our message-id if exists
  if (e->env->message_id) sprintf (tmp,"%s",e->env->message_id);
  else sprintf (tmp,"Message of %s from %s",e->env->date,frm);
  msg->in_reply_to = cpystr (tmp);

  sprintf (tmp,"On %s, %s wrote:\n\n> Subject: %s\n",
	   e->env->date,frm,e->env->subject ? e->env->subject : "(none)");
  write_address ((d = tmp),"> To:",e->env->to);
  write_address (d,"> cc:",e->env->cc);
  strcat (d,">\n> ");		// trailing junk
				// get size of header + text
  i = strlen (src = text) + strlen (tmp);
				// plus space for leading widgits in string
  while (*src) if (*src++ == '\n') i += 2;
  strcpy (d = fs_get (i+1),tmp);// start with header
  dst = d + strlen (d);		// set destination at end of header
  src = text;			// set up for copy
  while (c = *src++) {		// copy string, inserting brokets at each line
    if (c != '\r') *dst++ = c;	// first copy the character (nuke CR if seen)
    if ((c == '\n') && *src) {	// if newline and not end
      *dst++ = '>';		// insert widgit + space if not blank line
      if (*src != '\n') *dst++ = ' ';
    }
  }
				// back over blank lines
  while ((dst[-1] == '\n') && (dst[-2] == '>') && (dst[-3] == '\n')) dst -= 2;
				// ensure ending with newline
  if (dst[-1] != '\n') *dst++ = '\n';
  *dst++ = '\0';		// tie off string
  [NXApp setPasteboard:d];	// write reply message to pasteboard
  fs_give ((void **) &d);	// return the space
  return [self updateEnvelopeView];
}

// "Attach" on bottom of read window

- attachWindow:w
{
  NXRect frame;
  NXPoint pt;
  [window getFrame:&frame];	// get our dimensions
  pt.x = 0;			// get window's bottom left corner
  pt.y = 0;
  [w convertBaseToScreen:&pt];	// get it in screen coordinates
				// move our top left to that place
  [window moveTopLeftTo:pt.x :max (pt.y,NX_HEIGHT (&frame))];
  return self;
}


// Reply has been sent

- exit:sender
{
  char tmp[TMPLEN];
  MAILSTREAM *stream = handle ? mail_stream (handle) : NIL;
  if (stream && elt->msgno) {
				// mark message as answered
    sprintf (tmp,"%d",elt->msgno);
    mail_setflag (stream,tmp,"\\Answered");
				// update status in main window
    [(getstreamprop (stream))->window updateTitle];
  }
  [super exit:self];		// do the superior's actions
  return self;
}


// Window is closing

- windowWillClose:sender
{
  mail_free_elt (&elt);		// flush the cache element
  mail_free_handle (&handle);	// flush the MAIL stream handle
				// do the superior's actions
  return [super windowWillClose:sender];
}


@end
