/*
 * Program:	Distributed Electronic Mail Manager (SendWindow object)
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	24 February 1989
 *
 * Copyright (c) 1989, 1990, 1991 University of Washington
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */


#import "MailManager.h"

@implementation SendWindow

// Create a new SendWindow

+ new
{
  char tmp[TMPLEN];
  char *user = *username ? username :
    ((lastlogin && *lastlogin) ? lastlogin : localuser);
  self = [super new];		// create ourselves
				// get the interface
  [NXApp loadNibSection:"SendWindow.nib" owner:self withNames:NIL];
				// get handle on address book
  book = [NXApp addressBook:NIL];
  select = T;			// OK to call selectBody in actions
  env = mail_newenvelope ();	// instantiate the envelope
  body = mail_newbody ();	// instantiate the body
  env->remail = NIL;		// not a remail (yet)
  rfc822_date (tmp);		// get the date now
  env->date = cpystr (tmp);	// set up date string
  env->subject = NIL;		// initially no subject
				// calculate Return-Path:
  sprintf (tmp,"%s@%s",user,domainname);
  env->return_path = mtp_parse_address (tmp,(char *) domainname);
				// calculate Sender:
  if (*localpersonal) sprintf (tmp,"%s <%s@%s>",
			       localpersonal,localuser,localhost);
  else sprintf (tmp,"%s@%s",localuser,localhost);
  env->sender = (strcmp (user,localuser) ||
		 strcmp (domainname,localhost)) ?
		   mtp_parse_address (tmp,localhost) : NIL;
				// calculate From: field
  if (*personalname)
    sprintf (tmp,"%s <%s@%s>",personalname,user,domainname);
  else sprintf (tmp,"%s@%s",user,domainname);
  env->from = mtp_parse_address (tmp,(char *) domainname);
  env->reply_to = env->to = env->cc = env->bcc = NIL;
				// generate message ID
  sprintf (tmp,"<%s.%d.%d.%s@%s>",[NXApp appName],
	   time (0),rand () % 10000,localuser,localhost);
  env->message_id = cpystr (tmp);
  env->in_reply_to = NIL;
				// set TAB actions
  if (from) [from setNextText:[replyTo setNextText:[to setNextText:
	     [cc setNextText:[bcc setNextText:[subject setNextText:from]]]]]];
  else [to setNextText:[cc setNextText:[subject setNextText:to]]];
  [self updateEnvelopeView];	// set the envelope views from msg
  if (defaultcc && *defaultcc)
    [self setAddress:[cc setStringValue:defaultcc] :&env->cc];
  if (defaultbcc && *defaultbcc)
    [self setAddress:[bcc setStringValue:defaultbcc] :&env->bcc];
  [window makeKeyWindow];	// make us the key window
  return self;			// give us back to caller
}

// Note our window

- setWindow:anObject
{
  window = anObject;		// note our window
  [window setDelegate:self];	// we're the window's delegate
  return self;
}


// Note our From

- setFrom:anObject
{
  [[(from = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}


// Note our ReplyTo

- setReplyTo:anObject
{
  [[(replyTo = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}


// Note our To

- setTo:anObject
{
  [[(to = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}


// Note our Cc

- setCc:anObject
{
  [[(cc = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}

// Note our bcc

- setBcc:anObject
{
  [[(bcc = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}


// Note our Subject

- setSubject:anObject
{
  [[(subject = anObject) setFont:defaultfont] setTextDelegate:self];
  return self;
}


// Note our envelope view

- setEnvelopeView:anObject
{
  [(envelopeView = anObject) getFrame:&envelopeFrame];
  return self;
}


// Note our bodyView

- setBodyView:anObject
{
  NXRect frame;
				// note our view and get its frame
  [(bodyView = [anObject docView]) getFrame:&frame];
				// reset its font
  [bodyView renewFont:defaultfont text:"" frame:&frame tag:0];
  [bodyView setDelegate:self];	// we're our own delegate
  [bodyView setSelectable:T];	// we can select text
  [bodyView setEditable:T];	// the body view is editable
				// set up initial selection
  [bodyView setSel:(start = 0) :(end = 0)];
  return self;
}

// Note our attachment panel

- setAttachmentPanel:anObject
{
  attachmentPanel = anObject;	// note our attachment panel
  return self;
}


// Note our attachment view

- setAttachmentView:anObject
{
  NXSize size;
  NXRect frame;
  [anObject getFrame:&frame];	// get frame of attachments view
				// create a scroll view
  [(attachmentView = [ScrollView newFrame:&frame]) setBorderType:NX_BEZEL];
				// only vertical scrolling is required
  [attachmentView setVertScrollerRequired:T];
				// allow its size to change
  [attachmentView setAutosizing:NX_HEIGHTSIZABLE|NX_WIDTHSIZABLE];
				// put up the view
  [[anObject superview] replaceSubview:anObject with:attachmentView];
				// get the size minus the scroll bars
  [attachmentView getContentSize:&size];
				// set it as such in the browser frames
  NX_WIDTH (&frame) = size.width = 1300.0; NX_HEIGHT (&frame) = size.height;
				// create the browser
  browser = [Matrix newFrame:&frame mode:NX_RADIOMODE
	     cellClass:[ActionCell class] numRows:0 numCols:0];
  size.height = 13.0;		// large enough to hold the text
  [browser setCellSize:&size];	// set size of each cell
  size.width = size.height = 0;	// no intercell spacing
  [browser setIntercell:&size];
  [browser setAutoscroll:T];	// make it autoscrolling
  [[browser setAction:@selector(deleteAttachment:)] setTarget:self];
				// put the browser in the scroller and display
  [[attachmentView setDocView:browser] display];
  return self;
}

// Non-interface methods


// Return bodyView

- bodyView
{
  return bodyView;		// return the bodyView
}


// Return envelope for window

- (ENVELOPE *) msg
{
  return env;			// return the envelope
}


// Return window

- window
{
  return window;		// return the window
}

// Open the send help panel

- help:sender
{
  [NXApp sendHelp:sender];
  return self;
}


// From: field changed

- from:sender
{
  return [self setAddress:from :&env->from];
}


// Reply-To: field changed

- replyTo:sender
{
  return [self setAddress:replyTo :&env->reply_to];
}


// To: field changed

- to:sender
{
  return [self setAddress:to :&env->to];
}


// cc: field changed

- cc:sender
{
  return [self setAddress:cc :&env->cc];
}


// bcc: field changed

- bcc:sender
{
  return [self setAddress:bcc :&env->bcc];
}

// Set an address list

- setAddress:view :(ADDRESS **) list
{
  ADDRESS *adr;
  char tmp[16*TMPLEN];
  int i = strlen (domainname);
  if (view) {
				// flush old list
    if (*list) mail_free_address (list);
    memset (tmp,'@',i);		// flag for non-specified host
    tmp[i] = '\0';		// tie off string
				// get new address list
    if (*list = mtp_parse_address ((char *) [view stringValue],tmp)) {
				// if first address needs looking up
      if ((*list)->host[0] == '@') [book lookup:list];
      adr = *list;		// point at list here in case lookup: munged it
      while (adr->next) {	// lookup next address
	if (adr->next->host[0] == '@') [book lookup:&adr->next];
	adr = adr->next;	// try address after this one
      }
    }
    tmp[0] = '\0';		// start with empty string
				// reverse parse the list
    rfc822_write_address (tmp,*list);
				// set the new value
    [[view setStringValue:tmp] display];
  }
  if (select) [self selectBody];// select the body view
  return self;
}


// Subject: field changed

- subject:sender
{
  char *sub = (char *) [subject stringValue];
				// free any old subject
  if (env->subject) fs_give ((void **) &env->subject);
				// copy subject as new value
  if (strlen (sub)) env->subject = cpystr (sub);
  if (select) [self selectBody];// select the body view
  return self;
}


// Envelope item has lost first responder status

- textDidEnd:textObject endChar:(unsigned short) whyEnd
{
  id field = [textObject superview];
				// do action if tabbing
  if (textObject != bodyView && (whyEnd == NX_TAB || whyEnd == NX_BACKTAB)) {
    select = NIL;		// don't call selectBody, do the action
    [field sendAction:[field action] to:self];
    select = T;			// OK to call selectBody in actions again
  }
  return self;
}

// Body view is losing first responder status

- (BOOL) textWillEnd:textObject
{
  NXSelPt s,e;
  if (textObject == bodyView) {	// remember position if the body view
    [bodyView getSel:&s :&e];	// get current selection
    start = s.cp; end = e.cp;	// remember selection
  }
  return NIL;			// allow it
}


// Select the body view

- selectBody
{
				// select to if none there yet
  if (!env->to) return [to selectText:self];
				// select subject if none there yet
  if (!env->subject) return [subject selectText:self];
				// become first responder again
  [[bodyView setSel:start :end] display];
  return self;
}


// Update envelope views from msg

- updateEnvelopeView
{
  char tmp[16*TMPLEN];
  tmp[0] = '\0';		// update From
  rfc822_write_address (tmp,env->from);
  [[from setStringValue:tmp] display];
  tmp[0] = '\0';		// update Reply-To
  rfc822_write_address (tmp,env->reply_to);
  [[replyTo setStringValue:tmp] display];
  tmp[0] = '\0';		// update To
  rfc822_write_address (tmp,env->to);
  [[to setStringValue:tmp] display];
  tmp[0] = '\0';		// update cc
  rfc822_write_address (tmp,env->cc);
  [[cc setStringValue:tmp] display];
  tmp[0] = '\0';		// update bcc
  rfc822_write_address (tmp,env->bcc);
  [[bcc setStringValue:tmp] display];
				// update Subject
  [[subject setStringValue:env->subject] display];
  [self selectBody];		// put cursor back in the body view
  return self;
}

// Get text from a file

- insertFile:sender
{
  NXStream *file;
  char *text;
  int i,j;
  OpenPanel *query = [OpenPanel new];
  if ([query runModal]) {	// if got a file
				// try to open it
    if (!(file = NXMapFile ([query filename],NX_READONLY)))
      NXRunAlertPanel ("Open Failed","Can't open file",NIL,NIL,NIL);
    else {			// opened OK
				// get pointer to the text
      NXGetMemoryBuffer (file,&text,&i,&j);
				// insert the text
      [[bodyView replaceSel:text] display];
				// flush the file
      NXCloseMemory (file,NX_FREEBUFFER);
    }
  }
  return self;
}

// Attach a file

- attachFile:sender
{
  int i,j;
  char tmp[TMPLEN],tmpfile[TMPLEN];
  char c,*f,*s,*t;
  struct stat sbuf;
  NXStream *file;
  BODY *b = body;
  OpenPanel *query = [OpenPanel new];
  if ([query runModal] && (f = (char *) [query filename]) && !stat (f,&sbuf)) {
				// make file name for uuencode of attachment
    mktemp (strcpy (tmpfile,"/tmp/.AttachmentXXXXXX")); 
				// Unix command to create attachment
    sprintf (tmp,"compress -c '%s' | uuencode %s.Z > %s",f,
	     (s = strchr (f,'/') ? strrchr (f,'/') + 1 : f),tmpfile);
    if (system (tmp))		// make the attachfile file
      NXRunAlertPanel ("Attach Failed","Can't make attachment",NIL,NIL,NIL);
    else if (!(file = NXMapFile (tmpfile,NX_READONLY)))
      NXRunAlertPanel ("Attach Failed","Can't open attachment",NIL,NIL,NIL);
    else {			// get size of browser
      [browser getNumRows:&i numCols:&j];
      [browser insertRowAt:i];	// create a new row
      [[browser cellAt:i :0] setStringValue:f];
				// redraw the browser
      [[browser sizeToCells] display];
				// put up attachment panel if first time
      if (!i) [[attachmentPanel attachWindow:window] orderFront:self];
				// find end of body
      while (b->next) b = b->next;
				// instantiate a new body section
      (b = b->next = mail_newbody ())->type = ENCUUENCODE;
				// get text from file
      NXGetMemoryBuffer (file,&t,&i,&j);

				// build uuencode header with original mode
      sprintf (tmp,"begin %o %s.Z\015\012",(int) sbuf.st_mode & 0777,s);
				// attachment looks like our expectation?
      if (t[0] == 'b' && t[1] == 'e' && t[2] == 'g' && t[3] == 'i' &&
	  t[4] == 'n' && t[5] == ' ' && (s = strchr (t,'\n')) && (t = ++s))
	i = strlen (tmp);	// yes, note length of header
      else {			// wierd, use what we got
	s = t;			// copy from beginning of attachment
	tmp[i = 0] = '\0';	// no header
      }
      while (c = *s++)		// count bare line feeds
	((c == '\012') && i++) || ((c == '\015') && (*s == '\012') && s++);
				// create space for text
      b->text = (char *) fs_get (strlen (t) + i + 1);
				// copy header if needed
      s = tmp[0] ? strcpy (b->text,tmp) + strlen (tmp) : b->text;
      while (*t) {		// copy string, inserting CR's before LF's
				// if CR, copy it and skip LF check
        if (*t == '\015') *s++ = *t++;
				// else if LF, insert a CR before it
        else if (*t == '\012') *s++ = '\015';
        if (*t) *s++ = *t++;	// copy (may be null if last char was CR)
      }
      if (s[-1] != '\012') {	// force ending with a newline
	*s++ = '\015';
	*s++ = '\012';
      }
      *s = '\0';		// tie off destination
				// flush the file
      NXCloseMemory (file,NX_FREEBUFFER);
      unlink (tmpfile);		// delete it
    }
  }
  return self;
}

// Delete an attachment

- deleteAttachment:sender
{
  BODY *die;
  BODY *prev = body;
  int i = [browser selectedRow];
  [browser removeRowAt:i andFree:NIL];
				// redraw the browser
  [[browser sizeToCells] display];
  while (i--) prev = prev->next;// find its predecessor
  die = prev->next;		// body section to die
  prev->next = die->next;	// excise it from the list
  die->next = NIL;
  mail_free_body (&die);	// delete this attachment
  return self;
}

// Reformat selected text

- reformat:sender
{
  BOOL dot;
  int i;
  char *text,*src,*dst;
  NXSelPt s,e;
  [bodyView getSel:&s :&e];	// get current selection
  start = s.cp; end = e.cp;	// get the points as integers
				// calculate length of selection
  if ((i = 1 + end - start) > 1) {
				// get enough space for the text in worst case
    text = dst = (char *) fs_get (i*2);
				// get text from the view
    [bodyView getSubstring:(src = text+i) start:start length:i-1];
    text[i*2-1] = '\0';		// tie off text
    while (*src) {		// go through the entire string
      if (*src == '\n')		// if a newline
	switch (src[1]) {	// sniff at the next character
	case '\n':		// consecutive newlines are never mauled!
	  while (*src == '\n') *dst++ = *src++;
	  break;		// (don't need to worry about resetting dot)
	case '\t':		// tabs...
	case ' ':		//  and spaces
	  *dst++ = *src++;	// are data breaks and not mauled
	  break;		// (don't need to worry about resetting dot)
	default:
	  *dst++ = ' ';		// change it to a space
	  if (dot) *dst++ = ' ';// insert a second space if saw a period
	  src++;		// go to next input character
	  break;
	}
      else {			// otherwise copy the character
	if (*src != '\015') *dst++ = *src;
	dot = (*src++ == '.');	// remember if last character was a period
      }
    }
    *dst = '\0';		// tie off destination
    [bodyView replaceSel:text];	// replace with reformatted string
    [bodyView display];
  }
  return self;
}

// Send the message

- send:sender
{
  char tmp[TMPLEN];
  char hdr[16*TMPLEN];
  char *s;
  int i,j;
  struct tm *t;
  struct timeb tb;
  FILE *file;
  SMTPSTREAM *stream;
  BODY *b;
				// if have an MTP connection
  if (stream = mtp_open ((char **) hostlist,[debugging intValue])) {
				// flush old text
    if (body->text) fs_give ((void **) &body->text);
				// get text from view
    body->text = gettext (bodyView);
				// send mail
    if (mtp_mail (stream,"MAIL",env,body)) {
				// have an outbox and not remail?
      if (*outbox && !env->remail) {
	if (!(file = fopen (outbox,"a")))
	  NXRunAlertPanel ("Open failed","Can't open outbox",NIL,NIL,NIL);
	else {
	  ftime (&tb);		// get time and timezone poop
	  t = localtime (&tb.time);
	  sprintf (tmp,"%2d-%s-%02d %02d:%02d:%02d-%s",
		   t->tm_mday,months[t->tm_mon],t->tm_year,t->tm_hour,
		   t->tm_min,t->tm_sec,timezone (tb.timezone,t->tm_isdst));
				// calculate header
	  rfc822_header (hdr,env,body);
				// blat it to the file
	  append_msg (file,env->from,tmp,hdr,body);
	  fclose (file);	// close off the file
	}
      }
      [self exit:self];		// all done
    }
    else {			// otherwise report recipient errors
      [[[self rcptErr:env->to] rcptErr:env->cc] rcptErr:env->bcc];
      NXRunAlertPanel ("Sending Failed",stream->reply,NIL,NIL,NIL);
    }
    mtp_close (stream);		// nuke the stream in any case
  }
  else NXRunAlertPanel ("Sending Failed",
			"Unable to connect to server - try again",NIL,NIL,NIL);
  return self;
}

// Exit after success (intended to be overridden by ReplyWindow subclass)

- exit:sender
{
  [sender windowWillClose:self];// do abort actions
  [window close];		// close the window
  window = NIL;
  return self;
}


// Window is closing

- windowWillClose:sender
{
  [attachmentPanel close];	// nuke attachment panel
  attachmentPanel = NIL;
  mail_free_envelope (&env);	// flush the message
  mail_free_body (&body);
  return self;
}


// Window is resizing

- windowWillResize:sender toSize:(NXSize *) frameSize
{
				// constrain height to minimum
  frameSize->height = max (frameSize->height,60+NX_HEIGHT (&envelopeFrame));
  return self;
}


// Window moved

- windowDidMove:sender
{
				// make attachment panel track this window
  [attachmentPanel attachWindow:window];
  return self;
}

// Note our debugging switch

- setDebugging:anObject
{
  debugging = anObject;		// note our debug switch
  [debugging setIntValue:debug];// set its initial value
  return self;
}


// Report error in recipient list

- rcptErr:(ADDRESS *) adr
{
  char tmp[TMPLEN];
  if (adr) do if (adr->error) {	// if recipient had a problem
      sprintf (tmp,"%s@%s: %s",adr->mail_box,adr->host,adr->error);
				// log it (use telemetry window if one)
      mm_log (tmp,(lockedread ? ERROR : WARN));
    } while (adr = adr->next);	// try next recipient
  return self;
}


@end
