/*
 * Program:	Berkeley mail (BEZERK) definitions
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	20 December 1989
 *
 * Copyright 1989, 1990, 1991 by University of Washington.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */

/* Build parameters */

#define LOCKTIMEOUT 5		/* lock timeout in minutes */


/* Command bits from bezerk_getflags() */

#define fSEEN 1
#define fDELETED 2
#define fFLAGGED 4
#define fANSWERED 8


/* Status string */

#define STATUS "Status: %sO\nX-Status: %s%s%s\n\n"

/* BEZERK per-message cache information */

typedef struct file_cache {
  long internal;		/* pointer to internal header */
  long header;			/* pointer to RFC 822 header */
  long headersize;		/* size of RFC 822 header */
  long body;			/* pointer to message body */
  long bodysize;		/* size of message body */
  int date;			/* integer representation of date */
  char status[sizeof (STATUS)];	/* status flags */
} FILECACHE;


/* BEZERK I/O stream local data */

typedef struct bezerk_local {
  char *host;			/* local host name */
  char *fname;			/* local copy of file name */
  int ld;			/* lock file descriptor */
  char *lname;			/* lock file name */
  off_t filesize;		/* file size parsed */
  time_t filetime;		/* last file time */
  FILECACHE *msgs[MAXMESSAGES];	/* pointers to message-specific information */
  char *text;			/* mailbox text buffer */
  long textsize;		/* size of mailbox text buffer */
  long textend;			/* end of text in mailbox text buffer */
  char *lsjumb;			/* the Incomparables */
  char *buf;			/* temporary buffer */
  long buflen;			/* current size of temporary buffer */
} BEZERKLOCAL;


/* Convenient access to local data */

#define LOCAL ((BEZERKLOCAL *) stream->local)

/* Function prototypes */

DRIVER *bezerk_valid  ();
MAILSTREAM *bezerk_open  ();
void bezerk_close  ();
void bezerk_fetchfast  ();
void bezerk_fetchflags  ();
ENVELOPE *bezerk_fetchenvelope  ();
char *bezerk_snarf  ();
char *bezerk_fetchheader  ();
char *bezerk_fetchtext  ();
char *bezerk_fetchbody  ();
void bezerk_setflag  ();
void bezerk_clearflag  ();
void bezerk_search  ();
MAILSTREAM *bezerk_ping  ();
void bezerk_check  ();
void bezerk_expunge  ();
void bezerk_copy  ();
void bezerk_move  ();

void bezerk_abort  ();
char *bezerk_file  ();
int bezerk_lock  ();
void bezerk_unlock  ();
int bezerk_parse  ();
void bezerk_save  ();
int bezerk_writev  ();
void bezerk_copy_messages  ();
void bezerk_write_message  ();
void bezerk_update_status  ();
short bezerk_getsequence  ();
short bezerk_getflags  ();
char bezerk_search_all  ();
char bezerk_search_answered  ();
char bezerk_search_deleted  ();
char bezerk_search_flagged  ();
char bezerk_search_keyword  ();
char bezerk_search_new  ();
char bezerk_search_old  ();
char bezerk_search_recent  ();
char bezerk_search_seen  ();
char bezerk_search_unanswered  ();
char bezerk_search_undeleted  ();
char bezerk_search_unflagged  ();
char bezerk_search_unkeyword  ();
char bezerk_search_unseen  ();
char bezerk_search_before  ();
char bezerk_search_on  ();
char bezerk_search_since  ();
char bezerk_search_body  ();
char bezerk_search_subject  ();
char bezerk_search_text  ();
char bezerk_search_bcc  ();
char bezerk_search_cc  ();
char bezerk_search_from  ();
char bezerk_search_to  ();
char (*bezerk_search_date ())  ();
int bezerk_date  ();
char (*bezerk_search_flag ())  ();
char (*bezerk_search_string ())  ();
