/*
 * Program:	Interactive Mail Access Protocol 2 (IMAP2) definitions
 *
 * Author(s):	Mark Crispin
 *		Symbolic Systems Resources Group
 *		Knowledge Systems Laboratory
 *		Departments of Computer Science and Medicine
 *		Stanford University
 *		Stanford, CA  94305
 *
 * Date:	15 June 1988
 *
 * Sponsorship:	This work was supported by the Biomedical Research
 *		Technology Program of the National Institutes of Health
 *		under grant number RR-00785.
 *
 * Copyright 1988, 1989, 1990, 1991 by The Leland Stanford Junior University.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license from the Stanford Office of
 * Technology Licensing.
 *
 */


/* Build parameters */

#define MAXLOGINTRIALS 3	/* maximum number of login trials */
#define MAPLOOKAHEAD 20		/* fetch lookahead */


/* Constants */

#define IMAPTCPPORT 143		/* assigned TCP contact port */

/* IMAP2 specific definitions */


/* Parsed reply message from imap_reply */

typedef struct imap_parsed_reply {
  char *line;			/* original reply string pointer */
  char *tag;			/* command tag this reply is for */
  char *key;			/* reply keyword */
  char *text;			/* subsequent text */
} IMAPPARSEDREPLY;


/* IMAP2 I/O stream local data */

typedef struct imap_local {
  void *tcpstream;		/* TCP I/O stream */
  IMAPPARSEDREPLY reply;	/* last parsed reply */
  unsigned int use_encoding : 1;/* server supports encodings */
  unsigned int use_find : 1;	/* server supports FIND command */
  unsigned int use_bboard : 1;	/* server supports BBOARD command */
  char *header[MAXMESSAGES];	/* cached message headers */
  char *text[MAXMESSAGES];	/* cached message RFC822 texts */
} IMAPLOCAL;


/* Convenient access to local data */

#define LOCAL ((IMAPLOCAL *) stream->local)


/* Helper macro to take an action and return if two strings are equal */

#define IFEQUAL(S1,S2,ACTION) if (!strcmp (S1,S2)) { ACTION; return; }

/* Coddle certain compilers' 6-character symbol limitation */

#ifdef __COMPILER_KCC__
#define map_valid ivalid
#define map_open iopen
#define map_close iclose
#define map_fetchfast iffast
#define map_fetchflags ifflags
#define map_fetchenvelope ifenv
#define map_fetchheader ifhead
#define map_fetchtext iftext
#define map_fetchbody ifbody
#define map_setflag isflag
#define map_clearflag icflag
#define map_search isearch
#define map_ping iping
#define map_check icheck
#define map_expunge iexpun
#define map_copy icopy
#define map_move imove

#define imap_hostfield imhfld
#define imap_mailboxfield immfld
#define imap_host imhost
#define imap_select imsele
#define imap_send imsend
#define imap_reply imrepl
#define imap_parse_reply imprep
#define imap_fake imfake
#define imap_OK imok
#define imap_parse_unsolicited impuns
#define imap_parse_flaglst impflg
#define imap_searched imsear
#define imap_expunged imexpu
#define imap_parse_data impdat
#define imap_parse_prop imppro
#define imap_parse_envelope impenv
#define imap_parse_adrlist impadl
#define imap_parse_address impadr
#define imap_parse_flags impfla
#define imap_parse_sys_flag impsfl
#define imap_parse_user_flag impufl
#define imap_parse_string impstr
#define imap_parse_number impnum
#define imap_parse_enclist impecl
#define imap_parse_encoding impenc
#define imap_parse_body impbod
#endif

/* Function prototypes */

DRIVER *map_valid  ();
MAILSTREAM *map_open  ();
void map_close  ();
void map_fetchfast  ();
void map_fetchflags  ();
ENVELOPE *map_fetchenvelope  ();
char *map_fetchheader  ();
char *map_fetchtext  ();
char *map_fetchbody  ();
void map_setflag  ();
void map_clearflag  ();
void map_search  ();
MAILSTREAM *map_ping  ();
void map_check  ();
void map_expunge  ();
void map_copy  ();
void map_move  ();

char *imap_hostfield  ();
char *imap_mailboxfield  ();
char *imap_host  ();
IMAPPARSEDREPLY *imap_send  ();
IMAPPARSEDREPLY *imap_reply  ();
IMAPPARSEDREPLY *imap_parse_reply  ();
IMAPPARSEDREPLY *imap_fake  ();
int imap_OK  ();
void imap_parse_unsolicited  ();
void imap_parse_flaglst  ();
void imap_searched  ();
void imap_expunged  ();
void imap_parse_data  ();
void imap_parse_prop  ();
void imap_parse_envelope  ();
ADDRESS *imap_parse_adrlist  ();
ADDRESS *imap_parse_address  ();
void imap_parse_flags  ();
void imap_parse_sys_flag  ();
void imap_parse_user_flag  ();
char *imap_parse_string  ();
unsigned long imap_parse_number  ();
void imap_parse_enclist  ();
BODY *imap_parse_encoding  ();
void imap_parse_body  ();
