/*
 * Program:	Tenex mail (TENEX) definitions
 *
 * Author(s):	Mark Crispin
 *		Computing And Communications
 *		University of Washington
 *		Seattle, WA  98195
 *
 * Date:	22 May 1990
 *
 * Copyright 1990 by University of Washington.
 *
 * This program may be distributed without restriction for non-commercial use.
 * Any sale or use of this program or adaptations thereof for commercial
 * purposes is prohibited except under license.
 *
 */

/* Build parameters */


/* Command bits from tenex_getflags(), must correspond to mailbox bit values */

#define fSEEN 1
#define fDELETED 2
#define fFLAGGED 4
#define fANSWERED 8


/* TENEX per-message cache information */

typedef struct file_cache {
  long internal;		/* pointer to internal header */
  long header;			/* pointer to RFC 822 header */
  long headersize;		/* size of RFC 822 header */
  long body;			/* pointer to message body */
  long bodysize;		/* size of message body */
  long date;			/* integer representation of date */
} FILECACHE;


/* TENEX I/O stream local data */

typedef struct tenex_local {
  char *host;			/* local host name */
  short inbox;			/* if this is an INBOX or not */
  int fd;			/* file descriptor for I/O */
  off_t filesize;		/* file size parsed */
  FILECACHE *msgs[MAXMESSAGES];	/* pointers to message-specific information */
  char *text;			/* mailbox text buffer */
  long textsize;		/* size of mailbox text buffer */
  long textend;			/* end of text in mailbox text buffer */
  char *buf;			/* temporary buffer */
  long buflen;			/* current size of temporary buffer */
} TENEXLOCAL;


/* Convenient access to local data */

#define LOCAL ((TENEXLOCAL *) stream->local)

/* Function prototypes */

DRIVER *tenex_valid  ();
MAILSTREAM *tenex_open  ();
void tenex_close  ();
void tenex_fetchfast  ();
void tenex_fetchflags  ();
ENVELOPE *tenex_fetchenvelope  ();
char *tenex_fetchheader  ();
char *tenex_fetchtext  ();
char *tenex_fetchbody  ();
void tenex_setflag  ();
void tenex_clearflag  ();
void tenex_search  ();
MAILSTREAM *tenex_ping  ();
void tenex_check  ();
void tenex_expunge  ();
void tenex_copy  ();
void tenex_move  ();

char *tenex_file  ();
short tenex_getflags  ();
short tenex_parse  ();
void tenex_copy_messages  ();
MESSAGECACHE *tenex_elt  ();
void tenex_update_status  ();
short tenex_getsequence  ();
char tenex_search_all  ();
char tenex_search_answered  ();
char tenex_search_deleted  ();
char tenex_search_flagged  ();
char tenex_search_keyword  ();
char tenex_search_new  ();
char tenex_search_old  ();
char tenex_search_recent  ();
char tenex_search_seen  ();
char tenex_search_unanswered  ();
char tenex_search_undeleted  ();
char tenex_search_unflagged  ();
char tenex_search_unkeyword  ();
char tenex_search_unseen  ();
char tenex_search_before  ();
char tenex_search_on  ();
char tenex_search_since  ();
char tenex_search_body  ();
char tenex_search_subject  ();
char tenex_search_text  ();
char tenex_search_bcc  ();
char tenex_search_cc  ();
char tenex_search_from  ();
char tenex_search_to  ();
char (*tenex_search_date ())  ();
int tenex_date  ();
char (*tenex_search_flag ())  ();
char (*tenex_search_string ())  ();

char *bezerk_snarf  ();
